#ifndef XSTABTYP_H
#define XSTABTYP_H

/*
  Datei     : XStabTyp.h
  erstellt  : 25.06.96
  gendert  : 24.03.97
  Autor     : jh
*/

#define XS_LIMIT 1e-10
#define XS_EPS 0.05

#define XS_LAGER_X 1
#define XS_LAGER_Y 2   

#ifndef PI
#define PI 3.141592654
#endif


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <time.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <unistd.h>
#include <malloc.h>
#include <X11/xpm.h>
#include <forms.h> 

#include "FlgUsr.h"
#include "MngUsr.h"
#include "PmUsr.h"
#include "PslUsr.h"
#include "DsUsr.h"

#include "gui.h" 
#include "guiexp.h" 
#include "guilast.h" 
#include "guiinfo.h"
#include "guipfade.h"
#include "guicontrol.h"
#include "guidraw.h"
#include "gui_tabellen.h"
#include "gui_scale.h"
#include "gui_color.h"
#include "gui_freigrad.h"
#include "gui_geotopo.h"
#include "gui_schnitt.h"
#include "gui_ergebnisse.h"
#include "gui_prozessor.h"
#include "gui_verschieben.h"
#include "gui_punkterzeugen.h"
#include "gui_matquer.h"


#ifdef _TOOLTIPS_
#include <tooltips.h>
#endif


typedef struct
{
  double vs;                /* Verschiebung */ 
  double system;
  double kraft;             /* Knotenlast   */
  double streckenlast;
  double schnittkraft;
  double off_x, off_y;
  FL_Coord win_w, win_h;
} TypMaszstab;


typedef struct
{
  int raster;       /* 0 - aus;  1 - eingeschaltet */
  double raster_x, raster_y;
  
  int fang;         /* 0 - aus;   1 - koord;  
		       2 - punkt; 3 - punkt/koord */
  double fang_x, fang_y;

  int ursprung;

} TypEnvironment;


typedef struct
{
  unsigned long bg;                /* 0 */
  unsigned long fg;
  unsigned long punkt;
  unsigned long punkt_nr;
  unsigned long stab;
  unsigned long stab_select;       /* 5 */
  unsigned long stab_nr;
  unsigned long freigrad_dual;
  unsigned long freigrad_primal;
  unsigned long streckenlast;
  unsigned long punktlast;         /* 10 */
  unsigned long lager;
  unsigned long normal;
  unsigned long quer;
  unsigned long moment;
  unsigned long achsen;            /* 15 */
  unsigned long verschiebung;
  unsigned long feder;

  FL_COLOR fl_bg;
  FL_COLOR fl_fg;
  FL_COLOR fl_punkt;
  FL_COLOR fl_punkt_nr;
  FL_COLOR fl_stab;
  FL_COLOR fl_stab_select;
  FL_COLOR fl_stab_nr;
  FL_COLOR fl_freigrad_dual;
  FL_COLOR fl_freigrad_primal;
  FL_COLOR fl_streckenlast;
  FL_COLOR fl_punktlast;  
  FL_COLOR fl_lager;
  FL_COLOR fl_normal;
  FL_COLOR fl_quer;
  FL_COLOR fl_moment;
  FL_COLOR fl_achsen;
  FL_COLOR fl_verschiebung;
  FL_COLOR fl_feder;
  
} TypColor;


typedef struct
{
  long l_id;        /* persistenter ID */
  char c_name[50];  /* Bezeichnung */
  double d_E;       /* E Modul */
  double d_mue;     /* Querkontraktion */
  double d_alpha;   /* Waermeausdehnkoeff. */

} TypMaterial;



typedef struct
{
  long l_id;         /* persistenter ID */
  char c_name[50];   /* Bezeichnung */
  double d_A;        /* Flche */
  double d_Iz;       /* Fl-Traegheitsmoment um z */
  double d_Iy;       /* Fl-Traegheitsmoment um y */
  double d_maxy;     /* max z Koord. */

} TypQuerschnitt;



typedef struct
{
  long l_id;           /* persistenter ID */
  char c_datei[256];   /* Bezeichnung der Datei */

} TypSystem;



typedef struct
{
  long l_sysid;         /* System ID */
  long l_nummer;        /* fortlaufender ID */
  double d_x,d_y;

} TypPunkt;



typedef struct
{
  long l_id;           /* kleine Nummern ! */
  long l_aktiv;        /* LF aktiv == 1, inaktiv == 0 */
  char c_name[50];
  Menge *LFKomb;

} TypLastfallkombination;



typedef struct
{
  long l_id;
  double d_gewicht;

} TypLFKomb;



typedef struct
{
  long l_id;           /* persistent */
  long l_nummer;       /* wird zur Laufzeit vergeben */
  long l_aktiv;        /* LF aktiv == 1, inaktiv == -1 */
  double d_gewicht;
  char c_name[50];

} TypLastfall;



typedef struct
{
  long l_sysid;           /* ID des Systems */
  long l_nummer;          /* fortlaufende Nummer des FG */
  double d_verschiebung;
  double d_einwirkung;    /* aeussere, innere Einwirkung  */
  double d_auflager;      /* Trennung ist wichtig fuer das Zeichnen der Knotenlasten */
  double d_feder;         /* Federsteifigkeit */
  long l_index;           /* Index fuer das Prozessormodell */
  long l_status;          /* Primal; Dual  */
  long l_art;             /* 0->x  ;  1->y  ; 2->z */
  double d_alpha;         /* Drehung des Freiheitsgrades global */

} TypFreiGrad;



#include "XStabTyp_Nm.h"
#include "XStabTyp_Km.h"

 
typedef void (* const ok_fun) (FILE*);
typedef void (* const options_fun) (void);


typedef struct 
{
  ok_fun ok;
  options_fun options;
  char name[100];
  char extension[10];
} TypExport;


typedef struct
{
  TypSystem System;
  TypMaszstab Maszstab;
  TypEnvironment Environment;

  KeGlied *KePunkt_Nm;
  KeGlied *KePunkt_Km;

  KeGlied *KeStab_Nm;
  KeGlied *KeStab_Km;

  KeGlied *KeFreiGrad_Nm;
  KeGlied *KeFreiGrad_Km;

  KeGlied *KeLast;
  KeGlied *KePunktlast_Nm;
  KeGlied *KeStreckenlast_Nm;
  KeGlied *KeLfKombination;

  Menge *FG_Relation_Nm;

  int unsaved_changes;  
  long prozessor_anz;  /* Anzahl der Stabteilungen im Kernmodell */

  long nummer; /* Nummer in system_choice */

} TypModell;


#endif
