/*
   Datei : kette.c
   Autor : MF,ST
*/

#include "FlgUsr.h"


/*- Kette definieren und loeschen ----------------------------------------*/
KeGlied* KeKetteDefinieren (void)
{
  KeGlied *W ;
  
  /*- Grundelement der Kette wird allociert --------------------------------*/
  if((W = (KeGlied*) calloc (1, sizeof (KeGlied)))== NULL)
    {
      fprintf(stderr,"\a\a Fehler Nr. %ld bei Alloc\nKetteDefinieren\n",
	      errno);
      fprintf(stderr,
	      "Fehlercode ist in /usr/include/sys/errno.h definiert.\n");
      return NULL;
    }
  
  /*- Nachfolger und Vorgaenger werden auf Grundelement gesetzt ------------*/
  W->vor = W->nach = W ;
  W->x = NULL     ;
  
  return W ;
}

/*- Elemente der Kette austragen und Speicherplatz freigeben -------------*/
void KeKetteLoeschen (KeGlied *W)
{
  if(W == NULL)
    {
      fprintf(stderr,"Fehler in KetteLoeschen aufgetreten.\n");
      fprintf(stderr,"Es wurde ein NULL-Zeiger uebergeben.\n");
      errno = ENOMEM;
      return;
    }
  
  while (W->nach != W)
    KeElementAustragen (W->nach) ;
  
  /*- Speicher fuer die Kette freigeben ------------------------------------*/
  free (W) ;
  errno = 0;
}

/*- Glied und Element eintragen ------------------------------------------*/
void KeElementEintragen (void *x, int rel, KeGlied *q)
{
  KeGlied *p ;
  
  /*- Ueberpruefen, ob der uebergebene Zeiger zulaessig ist ----------------*/
  if(q == NULL)
    {
      fprintf(stderr,"Fehler in KeElementEintragen aufgetreten.\n");
      fprintf(stderr,"Es wurde ein NULL-Zeiger uebergeben.\n");
      errno = ENOMEM;
      return;
    }
  
  /*- Speicherplatz fuer Glied allocieren ----------------------------------*/
  if((p = (KeGlied*) calloc (1,sizeof (KeGlied)))==NULL)
    {
      fprintf(stderr,
	      "\a\a Fehler Nr. %ld bei Alloc\nKeElementEintragen\n",errno);
      fprintf(stderr,
	      "Fehlercode ist in /usr/include/sys/errno.h definiert.\n");
      return;
    }
  
  /*- Glied in der Kette eintragen -----------------------------------------*/
  KeGliedEintragen (p, rel, q) ;
  p->x = x ;
}

void KeGliedEintragen (KeGlied *p, int rel, KeGlied *q)
{
  KeGlied *v, *n ;
  
  /*- Falls NULL-Element return --------------------------------------------*/
  if (p  == NULL || q == NULL)
    {
      fprintf(stderr,"Fehler in KeGliedEintragen aufgetreten.\n");
      fprintf(stderr,"Es wurde ein NULL-Zeiger uebergeben.\n");
      errno = ENOMEM;
      return;
    }
  
  /*- Glied als Nachfolger des Gliedes q eintragen -------------------------*/
  if (rel == Nach)
    {
      n = q->nach ;
      q->nach = p ;  n->vor = p ;
      p->nach = n ;  p->vor = q ;
    }
  else
    
    /*- Glied als Vorgaenger des Gliedes q eintragen -------------------------*/
    {
      v = q->vor ;
      q->vor = p ;  v->nach = p ;
      p->vor = v ;  p->nach = q ;
    }
}

/*- Glied und Element austragen ------------------------------------------*/
void* KeElementAustragen (KeGlied *p)
{
  void *x ;
  
  /*- Falls NULL-Element return --------------------------------------------*/
  if (p == NULL)
    {
      fprintf(stderr,"Fehler in KeElementAustragen aufgetreten.\n");
      fprintf(stderr,"Es wurde ein NULL-Zeiger uebergeben.\n");
      errno = ENOMEM;
      return NULL;
    }
  
  /*- Falls Glied = Grundelement der Kette return --------------------------*/
  if (p->nach == p)
    return NULL  ;
  
/*- Uebernehmen des Elementzeigers ---------------------------------------*/
  x = p->x ;
  
  /*- Glied austragen und Speicherplatz freigeben --------------------------*/
  KeGliedAustragen (p) ;
  free (p) ;
  
  return x ;
}

void KeGliedAustragen (KeGlied *p)
{
  KeGlied *v, *n ;
  
  /*- Falls NULL-Element return --------------------------------------------*/
  if (p == NULL)
    {
      fprintf(stderr,"Fehler in KeGliedAustragen aufgetreten.\n");
      fprintf(stderr,"Es wurde ein NULL-Zeiger uebergeben.\n");
      errno = ENOMEM;
      return;
    }
  
  /*- Glied austragen durch verketten des Vorgaengers mit Nachfolger -------*/
  v = p->vor ;  n = p->nach ;
  n->vor = v ;  v->nach = n ;
}


void KeElementEinsortieren (KeGlied *K, Rel (*Vergleich) (), void *x)
{
  KeGlied *q ;
  
  /*- Falls NULL-Element return --------------------------------------------*/
  if (K == NULL)
    {
      fprintf(stderr,"Fehler in KeElementEinsortieren aufgetreten.\n");
      fprintf(stderr,"Es wurde ein NULL-Zeiger uebergeben.\n");
      errno = ENOMEM;
      return;
    }
  
  /*- Rueckwaertsschleife ueber alle Elemente bis Bedingung erfuellt -------*/
  q = KeErstesGlied(K) ;
  while (q != KeEnde(K) && Vergleich (x,q->x) != Less)
    q = q->nach ;
  
  /*- Element an der gefundenen Stelle q eintragen -------------------------*/
  KeElementEintragen (x,Vor,q) ;
}



/*= Schnelles Sortieren ==================================================*/
/**************************************************************************/
/** void KeSchnellSortieren (KeGlied *K, Rel (*Vergleich) ())            **/
/**                                                                      **/
/**   void                          es wird kein Wert zurueckgegeben     **/
/**                                                                      **/
/**   KeGlied *K                    Zeiger auf die zu sortierende Kette  **/
/**   Rel (*Vergleich) ()           Zeiger auf die Vergleichsfunktion    **/
/**************************************************************************/

void KeSchnellSortieren (KeGlied *K, Rel (*Vergleich) ())
{

/*- Falls NULL-Element return --------------------------------------------*/
	if (K == NULL)
	{
	        fprintf(stderr,"Fehler in KeSchnellSortieren aufgetreten.\n");
	        fprintf(stderr,"Es wurde ein NULL-Zeiger uebergeben.\n");
	        errno = ENOMEM;
	        return;
	}

	KeMSort (KeAnfang(K), KeEnde(K), Vergleich) ;
}


/**************************************************************************/
/**  void KeMSort (KeGlied *pa, KeGlied *pe, Rel (*Vergleich) ())        **/
/**                                                                      **/
/**   void                     es wird kein Wert zurueckgegeben          **/
/**                                                                      **/
/**   KeGlied *pa              Zeiger auf den linken Rand in der Kette   **/
/**   KeGlied *pe              Zeiger auf den linken Rand in der Kette   **/
/**   Rel (*Vergleich) ()      Zeiger auf die Vergleichsfunktion         **/
/**************************************************************************/

void KeMSort (KeGlied *pa, KeGlied *pe, Rel (*Vergleich) ())
{
	KeGlied *pi, *pj, *pm, *p ;
	Rel rel ;

/*- Falls NULL-Element return --------------------------------------------*/
	if (pa == NULL || pe == NULL)
	{
	        fprintf(stderr,"Fehler in KeMSort aufgetreten.\n");
	        fprintf(stderr,"Es wurde ein NULL-Zeiger uebergeben.\n");
	        errno = ENOMEM;
	        return;
	}

/*- Folge mit keinem oder einem Glied ------------------------------------*/
	if (pa->nach == pe || pa->nach == pe->vor)
	        return ;

/*- Teile die Folge in zwei moeglichst gleich lange Teilfolgen L,R -------*/
/*- Bestimme das letzte Glied pi von L und das erste Glied pj von R ------*/
	for (pi=pa->nach, pj=pe->vor; ; )
	{
	        if (pi->nach != pj)
	                pi = pi->nach ;
	        else
	                break ;

	        if (pj->vor  != pi)
	                pj = pj->vor  ;
	        else
	                break ;
	}

/*- Setze ein Trennglied zwischen die Teilfolgen L und R -----------------*/
	if((pm = (KeGlied*) calloc (1,sizeof (KeGlied)))==NULL)
	{
	        fprintf(stderr,"\a\a Fehler Nr. %ld bei Alloc\nKeMSort\n",errno);
	        fprintf(stderr,
	        "Fehlercode ist in /usr/include/sys/errno.h definiert.\n");
	        return ;
	}


	KeGliedEintragen (pm, Nach, pi)            ;

/*- Sortiere die beiden Teilfolgen L und R -------------------------------*/
	KeMSort (pa, pm, Vergleich) ;
	KeMSort (pm, pe, Vergleich) ;

/*- Verschmelze die beiden sortierten Teilfolgen L und R -----------------*/
	for (pi=pa->nach; pi!=pm; pi=pi->nach)
	        for (pj=pm->nach; pj!=pe; pj=pj->nach)
	        {
	                rel = Vergleich (pj->x,pi->x) ;
	                if (rel == Less)
	                {
	                        p = pj ;
	                        pj = pj->vor ;
	                        KeGliedAustragen (p) ;
	                        KeGliedEintragen (p, Vor, pi) ;
	                }
	                else if (rel == Greater)
	                        break ;
	        }

/*- Entferne das Trennglied ----------------------------------------------*/
	KeGliedAustragen (pm) ;
	free (pm) ;
}



/***********************************************************/
/* int KeElementSuchen(void *x, KeGlied *q)                */
/***********************************************************/
/*
   Fkt sucht Element (*x) in der Kette q
   wenn vorhanden return = 1
   wenn nicht     return = 0
   as, 02.04.97
*/
/***********************************************************/

int KeElementSuchen(void *x, KeGlied *q)
{
  KeGlied* p;
  void* tmp_element;
  
  KeDurchlaufVorwaerts(p,q)
    {
      tmp_element = p->x;
      if(x == tmp_element)      /* wenn Element gefunden */
	return 1;
    }
  return 0;
}

