/************************************************************************
  						INCLUDES 
************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include "MngUsr.h"

/*..Menge definieren................................................*/
Menge* Mng_MengeDefinieren (void)
{  
  Menge *M ;
  M       = (Menge*) calloc (1, sizeof (Menge)) ;
  M->feld = (void**) calloc (1, sizeof (void*)) ;
  M->m = M->n = 0 ;
  return M ;
}

/*..Menge loeschen..................................................*/
void Mng_MengeLoeschen (Menge *M)
{   free (M->feld) ;  free (M) ;  }

void Mng_MengeUndElementeLoeschen (Menge *M)
{   while (M->n) free (M->feld [M->n --]) ;
    free  (M->feld) ;  free (M) ;
}
/*..Menge kopieren..................................................*/
Menge* Mng_MengeKopieren (Menge *Q)
{   Menge *M ;   int i ;
    M       = (Menge*) calloc (1     , sizeof (Menge)) ;
    M->feld = (void**) calloc (Q->m+1, sizeof (void*)) ;
    for (i=1; i<=Q->n; i++)    M->feld[i] = Q->feld[i] ;
    M->m = Q->m ;
    M->n = Q->n ;
    return M ;
}
/*..Element eintragen...............................................*/
void Mng_ElementEintragen (void *x, Menge *M)
{   if (M->n == M->m)
    {   if (M->m == 0) M->m  = 1 ;   else  M->m *= 2 ;
	M->feld = (void**) realloc (M->feld, (M->m+1)*sizeof (void*));
    }   M->feld  [++ M->n] = x ;
}
/*..Element austragen...............................................*/
void* Mng_ElementAustragen (Menge *M)
{   void *x ;
    if (M->n == 0) x = M->feld [M->n]    ;
    else           x = M->feld [M->n --] ;
    if (M->m/2 > M->n)
    {   M->m /= 2 ;
	M->feld = (void**) realloc (M->feld, (M->m+1)*sizeof (void*));
    }   return x ;
}
/*..Elementare Operation : Element suchen...........................*/
/*  Ergebnis             : Feldindex des Elementes oder 0           */
int Mng_ElementSuchen (void *x, Menge *M)
{   int i    ;
    i = M->n ;   M->feld[0] = x ;
    while (M->feld[i] != x) i-- ;
    return i ;
}
/*..Elementare Operation : Element zufuegen.........................*/
void Mng_ElementZufuegen  (void *x, Menge *M)
{   if (!Mng_ElementSuchen (x,M)) Mng_ElementEintragen (x,M) ;  }

/*..Elementare Operation : Element entfernen........................*/
void Mng_ElementEntfernen (void *x, Menge *M)
{   int i ;
    if (i = Mng_ElementSuchen (x,M))
    {   M->feld [i] = M->feld [M->n] ;
	Mng_ElementAustragen (M) ;
}   }

/*..Vergleichsoperation : A ist Teilmenge von B.....................*/
Mng_Bool Mng_teilmenge (Menge *A, Menge *B)
{   int i ;
    for (i=1; i<=A->n; i++)
	if (!Mng_ElementSuchen (A->feld[i],B)) return Mng_False ;
    return Mng_True ;
}


Mng_Bool Mng_Teilmenge (Menge *A, Menge *B)
{   return (A->n <= B->n && Mng_teilmenge (A,B)) ? Mng_True : Mng_False ; }


/*..Vergleichsoperation : A ist echte Teilmenge von B...............*/
Mng_Bool Mng_EchteTeilmenge (Menge *A, Menge *B)
{   return (A->n < B->n && Mng_teilmenge (A,B)) ? Mng_True : Mng_False ; }

/*..Vergleichsoperation : A ist gleich B............................*/
Mng_Bool Mng_GeicheMenge (Menge *A, Menge *B)
{   return (A->n == B->n && Mng_teilmenge (A,B)) ? Mng_True : Mng_False ; }

/*..Algebraische Operation : C = A und B............................*/
Menge* Mng_Durchschnitt (Menge *A, Menge *B)
{   Menge *C ;  void *a ;  int i ;
    C = Mng_MengeDefinieren () ;
    for (i=1; i<=A->n; i++)
    {   a = A->feld [i] ;
	if (Mng_ElementSuchen (a,B))  Mng_ElementEintragen (a,C) ;
    }   return C ;
}
/*..Algebraische Operation : C = A oder B...........................*/
Menge* Mng_Vereinigung (Menge *A, Menge *B)
{   Menge *C ;  void *b ;  int i ;
    C = Mng_MengeKopieren (A) ;
    for (i=1; i<=B->n; i++)
    {   b = B->feld [i] ;
	if (!Mng_ElementSuchen (b,A))  Mng_ElementEintragen (b,C) ;
    }   return C ;
}
/*..Algebraische Operation : C = A ohne B...........................*/
Menge* Mng_Differenz (Menge *A, Menge *B)
{   Menge *C ;  int i ;
    C = Mng_MengeKopieren (A) ;
    for (i=1; i<=B->n; i++)
	Mng_ElementEntfernen (B->feld[i], C) ;
    return C ;
}
