/*
  Datei     : psl.c
  erstellt  : 30.11.96
  geaendert : 02.12.96
  Autor     : jh, sm
*/

#include "math.h" 
#include "PslUsr.h"


void ps_trans(double x, double y, double *X, double *Y);


static FILE *pf = NULL;
static double psmass;
static double rotation=0;


void ps_open(FILE *f)
{
  pf = f;
  psmass = ps_mass();
  fprintf(pf,"%%!PS\n");      
}


void ps_begin_page(void)
{   
}


void ps_begin_page_bb(FILE *f,long llx, long lly, long urx, long ury)
{
  pf = f;
  fprintf(pf,"%%!PS\n");  
  if( (urx-llx) && (ury-lly))
    {
      fprintf(pf,"%%%%BoundingBox: %ld %ld %ld %ld\n",llx,lly,urx,ury);  
      fprintf(pf,"%%%%EndComments\n");
      fprintf(pf,"%%%%BeginProlog\n");
      fprintf(pf,"%%%%EndProlog\n");
      fprintf(pf,"%%%%Page: one 1\n");
      fprintf(pf,"%%%%BeginPageSetup\n");
      fprintf(pf,"%%%%EndPageSetup\n");
    }
  
}


void ps_direct(char *t) 
{
  fprintf(pf,"%s\n",t);
}


void ps_end_page(void) 
{
  fprintf(pf,"stroke\n");
  fprintf(pf,"showpage\n"); 
}


void ps_line(double x1, double y1, double x2, double y2) 
{
  double X,Y;

  ps_trans(x1,y1,&X,&Y);
  fprintf(pf, "%.3f %.3f moveto\n", X, Y);

  ps_trans(x2,y2,&X,&Y);
  fprintf(pf, "%.3f %.3f lineto\n", X, Y);

  fprintf(pf,"stroke\n");
}


void ps_rect(double x, double y, double width , double height)
{
  double X,Y;

  fprintf(pf,"newpath\n"); 

  ps_trans(x,y,&X,&Y);
  fprintf(pf, "%.3f %.3f moveto\n" , X, Y);

  ps_trans(x+width, y, &X, &Y);
  fprintf(pf, "%.3f %.3f lineto\n", X, Y);

  ps_trans(x+width, y+height, &X, &Y);
  fprintf(pf, "%.3f %.3f lineto\n", X, Y);

  ps_trans(x, y+height, &X, &Y);
  fprintf(pf, "%.3f %.3f lineto\n", X, Y);

  ps_trans(x,y,&X,&Y);
  fprintf(pf, "%.3f %.3f lineto\n", X, Y);

  fprintf(pf,"closepath stroke\n");
}


void ps_arc(double x, double y, double r)
{
  double X,Y;

  fprintf(pf,"newpath\n"); 

  ps_trans(x,y,&X,&Y);
  fprintf(pf, "%.3f %.3f %.3f 0 360 arc\n",X, Y, r*psmass);

  fprintf(pf,"closepath stroke\n");
}


void ps_arrow(double x1, double y1, double x2, double y2 )
{
  double laenge;
  double alpha=0;
 
  laenge = sqrt((x2-x1)*(x2-x1) + (y2-y1)*(y2-y1));

  if(x2 - x1) {
    alpha = atan((y2-y1)/(x2-x1)); 
    if( (y2-y1)==0 && (x2<x1))
      alpha = -M_PI; 
  }  
  else { 
    if(y1<y2) 
      alpha = +M_PI/2; 
    if(y1>y2) 
      alpha = -M_PI/2; 
  }   

  if( (x2-x1)<0 && (y2-y1)>0 ) 
    alpha += M_PI ;   

  if( (x2-x1)<0 && (y2-y1)<0 ) 
    alpha += M_PI ; 

  fprintf(pf,"% <arrow>\n");
  
  ps_line((double) x1, (double) y1, 
	  (double) x2, (double) y2);

  /* Spitze zeichnen */
  ps_line(x2, y2,
	  (double) ( x2 - ( laenge/6 * cos(alpha - M_PI/6)) ),
	  (double) ( y2 - ( laenge/6 * sin(alpha - M_PI/6)) ));
	    
  ps_line(x2, y2,
	  (double) ( x2 - ( laenge/6 * sin(M_PI/3 - alpha)) ),
	  (double) ( y2 - ( laenge/6 * cos(M_PI/3 - alpha)) ));	    

  fprintf(pf,"% </arrow>\n");
  
}



void ps_linewidth(double width) 
{ 
  fprintf(pf,"%f setlinewidth\n",width * psmass );
}


void ps_linestyle(long style) 
{
  switch (style)
    {   
    case 1 :
      fprintf(pf,"newpath [%f] 0 setdash\n",2 * psmass);
      break;

    case 2 :
      fprintf(pf,"newpath [%f %f] 0 setdash\n",3 * psmass, 2 * psmass);
      break;

    default :
      fprintf(pf,"newpath [] 0 setdash\n");
      break;
    }
}


void ps_scale (double x, double y) 
{
  fprintf(pf,"%f %f scale\n",x,y);
}


void ps_setgray(double percent) 
{
  fprintf(pf,"%.2f setgray\n",percent);
}


void ps_text_font (char *f, long size) 
{
  fprintf(pf,"/%s findfont\n",f); 
  fprintf(pf,"%ld scalefont setfont\n",size);
}


void ps_drawtext(double x, double y, char *t) 
{
  double X,Y;
  ps_trans(x,y,&X,&Y);

  fprintf(pf,"%f %f moveto \n",X,Y);
  fprintf(pf,"%f rotate\n",rotation*180/M_PI); 
  fprintf(pf,"(%s) show\n", t);
  fprintf(pf,"%f rotate\n",-(rotation*180/M_PI));
}


void ps_translate (double dx, double dy) 
{
  fprintf (pf,"%f %f translate\n",dx*psmass,dy*psmass);
}


void ps_rotate (double phi)
{
  rotation = phi * M_PI/180;
}


double ps_mass(void)
{
  /* ...  alle Angaben in mm  ... */
  return (1/0.3527778);
}


void ps_trans(double x, double y, double *X, double *Y)
{
  *X = psmass * (x * cos(rotation) - y * sin(rotation));
  *Y = psmass * (x * sin(rotation) + y * cos(rotation));
}
