/*
  Datei     : Erzeuge_Km.c
  erstellt  : 22.07.96
  gendert  : 14.04.97
  Autor     : jh,as
*/


#include "XStabDek.h"



/***********************************************************/ 
/* TypStab* Erzeuge_Stab_Km (void)                        */
/***********************************************************/ 
/*                                              
   Konstruktor fr einen Stab im Kernmodell
   jh, 16.03.97
*/
/***********************************************************/

TypStab_Km* Erzeuge_Stab_Km(long nummer)
{
  TypStab_Km *tmp_stab;

  if(!(tmp_stab = (TypStab_Km*) calloc (1,sizeof(TypStab_Km)))) {
    fprintf(stderr,"XStab : MemFault\n");
    exit(1);
  }
  
  /* Nummer des dazugehrigen Stabes im Nm */
  (*tmp_stab).l_nummer = nummer; 

  /* initialisieren */
  (*tmp_stab).d_lastp_a = 0;
  (*tmp_stab).d_lastp_b = 0;
  (*tmp_stab).d_lastn_a = 0;
  (*tmp_stab).d_lastn_b = 0;
  
  /* Stab in Kette eintragen */
  if(Modell->KeStab_Km)
    KeElementEintragen(tmp_stab,Nach,Modell->KeStab_Km);
  else {
    Modell->KeStab_Km = KeKetteDefinieren();
    KeElementEintragen(tmp_stab,Nach, Modell->KeStab_Km);
  } 
  
  return tmp_stab;

}



/***********************************************************/ 
/* void Erzeuge_FG_Km(TypStab_Km *tmp_stab, long fg_art)   */
/***********************************************************/ 
/*                                              
   Konstruktor fr Freiheitsgrad im Kernmodell
   fr tmp_stab (Kernmodell)  fg_art -> 0 - 2 
   0->x ; 1->y ; 2->z ;   
   nummer == Nummer im Nutzermodell
   nummer == -1 -> keine Zuordnung zum Nm
   as, 06.04.97
   jh, 14.04.97
*/
/***********************************************************/

TypFreiGrad* Erzeuge_FG_Km(long fg_art, long nummer)
{ 
  TypFreiGrad *tmp_freigrad;
  KeGlied *p;

  /* wenn nummer != -1, dann FG suchen */
  if(nummer!=-1) {
    if(Modell->KeFreiGrad_Km)
      KeDurchlaufVorwaerts(p,Modell->KeFreiGrad_Km) {
	tmp_freigrad = p->x;
	if((*tmp_freigrad).l_nummer == nummer)
	  return tmp_freigrad;
      }
  }
      
  if(!(tmp_freigrad = (TypFreiGrad*) calloc (1,sizeof(TypFreiGrad)))) {
    fprintf(stderr,"XStab : MemFault\n");
    exit(1);
  }
  
  /* wird nicht bentigt */
  (*tmp_freigrad).l_sysid = 0;

  /* wenn nummer != -1 -> FG ist aus dem Nutzermodell bernommen */  
  (*tmp_freigrad).l_nummer = nummer;
 
  /* Fg - Typ eintragen */
  (*tmp_freigrad).l_art = fg_art;    
 
  /* Initialisierung */
  (*tmp_freigrad).d_verschiebung = 0;
  (*tmp_freigrad).d_einwirkung  = 0;      
  (*tmp_freigrad).d_auflager  = 0;    
  (*tmp_freigrad).d_alpha = 0;         
  (*tmp_freigrad).d_feder = 0;

  if(Modell->KeFreiGrad_Km)    
    KeElementEintragen(tmp_freigrad, Nach, Modell->KeFreiGrad_Km);    
  else {
    Modell->KeFreiGrad_Km = KeKetteDefinieren();
    KeElementEintragen(tmp_freigrad, Nach, Modell->KeFreiGrad_Km);
  }
  
  return tmp_freigrad;
  
}



/***********************************************************/ 
/* TypPunkt* Erzeuge_Punkt_Km(double x, double y, long id) */
/***********************************************************/ 
/*                                              
   Konstruktor fr einen Punkt im Kernmodell
   wenn id == -1 , dann wird ein ID vergeben
   wenn id != -1 wird der id in (*tmp_punkt).l_nummer
   eingetragen
   as, 01.04.97
*/
/***********************************************************/

TypPunkt* Erzeuge_Punkt_Km(double x, double y)
{
  TypPunkt* tmp_punkt;
  KeGlied *p;
  

  /* berprfen, ob schon vorhanden */
  if(Modell->KePunkt_Km)
    KeDurchlaufVorwaerts(p, Modell->KePunkt_Km) {
      tmp_punkt = p->x;
      if((*tmp_punkt).d_x == x)
	if((*tmp_punkt).d_y == y)
	  return tmp_punkt;
    }


  /* ... Punkt im Km erzeugen  ... */
  if(!(tmp_punkt = (TypPunkt*) calloc (1,sizeof(TypPunkt)))) {
    fprintf(stderr,"XStab : MemFault\n");
    exit(1);
  }
   
  (*tmp_punkt).d_x = x;
  (*tmp_punkt).d_y = y;

  /* nummer und id werden nicht bentigt */
  (*tmp_punkt).l_nummer = 0;
  (*tmp_punkt).l_sysid = 0;
  
  /* Punkt in Kette eintragen */
  if(Modell->KePunkt_Km) 
    KeElementEintragen(tmp_punkt, Nach, Modell->KePunkt_Km);
  else {
    Modell->KePunkt_Km = KeKetteDefinieren();
    KeElementEintragen(tmp_punkt, Nach, Modell->KePunkt_Km);
  }
  
  return tmp_punkt;
}


