/*
  Datei     : staberzeugen_Km.c
  erstellt  : 22.03.97
  geaendert : 03.05.97
  Autor     : jh
*/


#include "XStabDek.h"


#define EPS_LAENGE 0.05
#define EPS_TEILLAENGE 0.005


/*********************************************************************/
/* void Stabteilen_Km(long anz, TypStab_Km *stab)                    */
/*********************************************************************/
/*                                              
   Funktion teilt einen Stab im Kernmodell in 'anz' Teile
   jh, 01.05.97 
*/
/*********************************************************************/

void Stabteilen_Km(long anz, TypStab_Km *stab)
{
  double step,x,y,z;
  double z1,z2;
  long i,j;
  TypStab_Km **tmp_stab = NULL;
  TypPunkt **tmp_punkt = NULL; 
  TypFreiGrad *tmp_fg = NULL;
  double lastp_a=0,lastp_b=0;
  double lastn_a=0,lastn_b=0;
  double posA,posB;

  double XA,YA,XE,YE;

  if(anz<2)
    return;

  if( (*stab).d_laenge < EPS_LAENGE )
    return;

  if( ((*stab).d_laenge / (double) anz) < EPS_TEILLAENGE )
    anz = (long) ( (*stab).d_laenge / EPS_TEILLAENGE );


  /* fprintf(stderr,"l : %lf\n",  (*stab).d_laenge ); */

  /* Position des alten Stabes sichern */
  posA = (*stab).d_posA;
  posB = (*stab).d_posB;

  /* Last sichern */
  lastp_a = (*stab).d_lastp_a;
  lastp_b = (*stab).d_lastp_b;
  lastn_a = (*stab).d_lastn_a;
  lastn_b = (*stab).d_lastn_b;

  /* Vektor, in dem die Stbe stehen, stab[0] ist der alte Stab */
  tmp_stab = (TypStab_Km**) calloc (anz,sizeof(TypStab_Km*));
  tmp_stab[0] = stab;

  /* Vektor, in dem die Punkte stehen */
  tmp_punkt = (TypPunkt**) calloc (anz+1,sizeof(TypPunkt*));

  /* ersten und letzten Punkt setzen */
  tmp_punkt[0]   = (*tmp_stab[0]).pe_punkta;
  tmp_punkt[anz] = (*tmp_stab[0]).pe_punktb;

  /* neue Staebe erzeugen */
  for(i=1;i<anz;i++)
    tmp_stab[i] = Erzeuge_Stab_Km((*stab).l_nummer);
  
  /* Geometrie sichern */
  XA = (*tmp_punkt[0]).d_x;
  YA = (*tmp_punkt[0]).d_y;
  XE = (*tmp_punkt[anz]).d_x;
  YE = (*tmp_punkt[anz]).d_y;

  /* neue Punkte erzeugen */
  step = (double) (2./(double) anz);
  z = -1.0 + step;
  for(i=1;i<anz;i++) {
    
    x = 0.5 * (XA * (1-z) + XE * (1+z));
    y = 0.5 * (YA * (1-z) + YE * (1+z));

    tmp_punkt[i] = Erzeuge_Punkt_Km(x,y);
    z+=step;

  }  

  /* Punkte den Staeben zuweisen */
  (*tmp_stab[0]).pe_punktb = tmp_punkt[1];

  for(i=1;i<anz;i++) {
    (*tmp_stab[i]).pe_punkta = tmp_punkt[i];
    (*tmp_stab[i]).pe_punktb = tmp_punkt[i+1];  
  } 

  /* Position im zugehrigen Stab im Nm neu berechnen */
  for(i=0;i<anz;i++) {
    z1 = (double) ((double) i/anz);
    z2 = (double) ((double) (i+1.)/anz);
    (*tmp_stab[i]).d_posA = posA * (1.- z1) + posB * z1;
    (*tmp_stab[i]).d_posB = posA * (1.- z2) + posB * z2;
  }

  /* die FG 3-6 des alten Stabes, dem letzten neuen Stab zuweisen */
  for(i=3;i<6;i++)
    (*tmp_stab[anz-1]).pe_frei[i] =  (*tmp_stab[0]).pe_frei[i];

  /* die FG 3-6 des alten Stabes neu erzeugen */
  for(i=3;i<6;i++) {
    (*tmp_stab[0]).pe_frei[i] = Erzeuge_FG_Km(i-3,-1);
    tmp_fg = (*tmp_stab[0]).pe_frei[i];
    (*tmp_fg).l_status = CPmDual;
  }


  for(i=1;i<anz-1;i++) {    
    for(j=0;j<3;j++)
      (*tmp_stab[i]).pe_frei[j] = (*tmp_stab[i-1]).pe_frei[j+3];
    for(j=3;j<6;j++) {
      (*tmp_stab[i]).pe_frei[j] = Erzeuge_FG_Km(j-3,-1);
      tmp_fg = (*tmp_stab[i]).pe_frei[j];
      (*tmp_fg).l_status = CPmDual;
    }
  } 

  for(i=0;i<3;i++)
    (*tmp_stab[anz-1]).pe_frei[i] = (*tmp_stab[anz-2]).pe_frei[i+3];  
  
  /* Geometrie neu berechnen */
  for(i=0;i<anz;i++) 
    SetzeGeometrie_Km(tmp_stab[i]);    
        
  for(i=1;i<anz;i++) {
    (*tmp_stab[i]).l_matid  = (*stab).l_matid;     
    (*tmp_stab[i]).l_querid = (*stab).l_querid;    
  }


  /* Last interpolieren */
  for(i=0;i<anz;i++) { 
    (*tmp_stab[i]).d_lastp_a = lastp_a * (1.-((double) i/anz)) + lastp_b * ((double) i/anz);
    (*tmp_stab[i]).d_lastn_a = lastn_a * (1.-((double) i/anz)) + lastn_b * ((double) i/anz);
    (*tmp_stab[i]).d_lastp_b = lastp_a * (1.-((double) (i+1.)/anz)) + lastp_b * ((double) (i+1.)/anz);
    (*tmp_stab[i]).d_lastn_b = lastn_a * (1.-((double) (i+1.)/anz)) + lastn_b * ((double) (i+1.)/anz);
  }

  /* Speicher freigeben */
  free(tmp_stab);
  free(tmp_punkt);

}


/*********************************************************************/
/* void SetzeGeometrie_Km(TypStab_Km* tmp_stab)                      */
/*********************************************************************/ 
/*                                              
   Funktion berechnet die abgeleiteten Geometriewerte fr
   einen Stab im Kernmodell
   jh,15.04.97                     
*/
/*********************************************************************/

void SetzeGeometrie_Km(TypStab_Km* tmp_stab)
{
  double x1,y1,x2,y2;
  TypPunkt *p1,*p2;

  p1 = (*tmp_stab).pe_punkta;
  p2 = (*tmp_stab).pe_punktb;

  x1 = (*p1).d_x;
  y1 = (*p1).d_y;

  x2 = (*p2).d_x;
  y2 = (*p2).d_y;

  if(x2-x1) {
    (*tmp_stab).d_alpha = atan((y2-y1)/(x2-x1)); 
    if( (y2-y1)==0 && (x2<x1))
      (*tmp_stab).d_alpha = -PI; 
  } 
  else {
    if(y1<y2)
      (*tmp_stab).d_alpha = PI/2.;
    if(y1>y2)
      (*tmp_stab).d_alpha = -PI/2.;
  }
  
  if( (x2-x1)<0 && (y2-y1)>0 )
    (*tmp_stab).d_alpha += PI ;
  
  if( (x2-x1)<0 && (y2-y1)<0 )
    (*tmp_stab).d_alpha += PI ;
  
  (*tmp_stab).d_laenge = sqrt( (x1-x2)*(x1-x2) + (y1-y2)*(y1-y2));
  (*tmp_stab).d_cos =  cos((*tmp_stab).d_alpha);
  (*tmp_stab).d_sin =  sin((*tmp_stab).d_alpha);      

}
