/*
  Datei     : hermite.c
  erstellt  : 12.05.97
  geaendert : 
  Autor     : jh
*/

   
#include "XStabDek.h"


/*********************************************************************/
/*  double* Hermite()                                                */
/*********************************************************************/ 
/*                                              
   Fkt interpoliert eine gegebene Anzahl von Sttzstellen
   in einem Intervall nach Hermite
   return : Vektor mit Sttzwerten
   jh, 15.03.97

*/
/*********************************************************************/

double* Hermite(double wa,  /* Fkt-Wert */
		double wsa, /* Ableitung an Punkt Anfang */
		double we,  /* Fkt-Wert */
		double wse, /* Ableitung an Punkt Ende */
		double l,   /* Intervallnge */
		long anz)   /* Anz. Sttzstellen */
{
  double *w;
  double z, step;
  long i; 

  w = (double*) calloc (anz,sizeof(double));

  wsa *= (l/2);
  wse *= (l/2);

  z = -1;
  step = (double) (2./((double) anz - 1.));
  
  for(i=0; i<anz; i++) {           	  	     
    w[i] = 0.25 * (wa  * (2 - 3*z + z*z*z) +
		   wsa * (1 - z - z*z + z*z*z) + 
		   we  * (2 + 3*z - z*z*z) +
		   wse * (-1 - z + z*z + z*z*z)); 
    z +=step;    
  }    
  
  return w;

}



/*********************************************************************/
/*  double Hermite_dis()                                             */
/*********************************************************************/ 
/*                                              
   Fkt interpoliert eine bestimmte Stelle
   in einem Intervall nach Hermite
   return : Sttzwerte an der Stelle z mit -1 < z < 1
   jh, 15.03.97

*/
/*********************************************************************/

double Hermite_dis(double wa, double wsa, 
		   double we, double wse, 
		   double l, double z)
{
  double w;

  wsa *= (l/2);
  wse *= (l/2);        	  	 
      
  w = 0.25 * (wa * (2 - 3*z + z*z*z) +
	      wsa * (1 - z - z*z + z*z*z) + 
	      we * (2 + 3*z - z*z*z) +
	      wse * (-1 - z + z*z + z*z*z));  

  return w;
}



/*********************************************************************/
/* double* Hermite_extrema( ... )                                    */
/*********************************************************************/ 
/*                                              
   Fkt berechnet die Extrema in einem Intervall nach Hermite
   return : Position der Extrema in normierten Koordinaten
   ( -1 < z < 1 )
   jh, 20.04.97
*/
/*********************************************************************/

double* Hermite_extrema(double wa, double wsa, 
			double we, double wse,
			double l)
{
  double a,b,c;
  double *z;

  wsa *= (l/2.);
  wse *= (l/2.);
  
  z = (double*) calloc (2,sizeof(double));

  a = 0.25 * (3.*wa + 3.*wsa - 3.*we + 3.*wse);
  b = 0.25 * (-2.*wsa + 2.*wse);
  c = 0.25 * (-3.*wa - wsa + 3.*we - wse);   

  /* b==0 -> Abl = const */
  if( b>-1e-10  && b<1e-10) {    
    free(z);      
    return NULL;
  }    

  /* a==0 -> Abl = lineare Fkt */
  if( a>-1e-10  && a<1e-10) {    
    z[0] = -c/b;
    z[1] = -c/b;
    return z;
  }    
  
  /* keine Lsg */
  if( (b*b - 4*a*c) < 0 ) {
    free(z);
    return NULL;
  }

  z[0] = (1./(2.*a)) * (-b + sqrt(b*b - 4.*a*c));
  z[1] = (1./(2.*a)) * (-b - sqrt(b*b - 4.*a*c)); 

  return z;
  
}

