/*
  Datei    : export_liste.c
  erstellt : 04.12.96
  gendert : 12.05.97
  Autor    : jh
  Bemerkung :
*/


#include "XStabDek.h"



/*********************************************************************/
/* void liste_ok(FILE *d)                                            */
/*********************************************************************/ 
/*                                              
   Funktion schreibt eine Liste aller Stbe in
   eine Datei
   jh, 21.03.97
*/
/*********************************************************************/

void liste_ok(FILE *d, KeGlied *staebe)
{
  long i,*id; 
  TypStab_Nm *tmp_stab; 
  TypLastfall *tmp_lastfall;
  double M,pos;
  Menge *M_aktiv=NULL;    


  if(!staebe)
    return;
 
  fprintf(d,"  -----------------------------------------------------------------------\n\r"); 
  fprintf(d,"   System : %s\n\r",Modell->System.c_datei);
  fprintf(d,"  -----------------------------------------------------------------------\n\r"); 
  fprintf(d,"\n\r");
  
  fprintf(d,"  aktive Lastflle : \n\r");
  fprintf(d,"\n\r");  

  /* Mengen und Ketten definieren */
  M_aktiv = Mng_MengeDefinieren();

  /* Bestimmen d. aktiven Lastflle */
  HoleAktiveLastfaelle_Lastwahl(M_aktiv); 

  /* Ausgabe der aktiven Lastflle */
  for(i=1;i<=M_aktiv->n;i++) {    
    id = M_aktiv->feld[i];
    tmp_lastfall = Suche_Lastfall_id(*id);
    if(tmp_lastfall)
      fprintf(d,"  Nr.:  %ld  %.2f x %s\n\r",
	      (*tmp_lastfall).l_nummer,
	      (*tmp_lastfall).d_gewicht, 
	      (*tmp_lastfall).c_name);
  }
  fprintf(d,"\n\r"); 

  /* Extrema im System */
  fprintf(d,"  Extrema im System :\n\r");
  fprintf(d,"\n\r");  

  M = maxMKette(staebe, &tmp_stab, &pos); 
  fprintf(d,"  Stab Nr : %3ld   max M : %8.2f kNm [ %5.2f m ]\n\r",
	  (*tmp_stab).l_nummer, M, pos * (*tmp_stab).d_laenge);

  M = maxQKette(staebe, &tmp_stab, &pos);
  fprintf(d,"  Stab Nr : %3ld   max Q : %8.2f kN  [ %5.2f m ]\n\r",
	  (*tmp_stab).l_nummer, M, pos * (*tmp_stab).d_laenge);

  M = maxNKette(staebe, &tmp_stab, &pos); 
  fprintf(d,"  Stab Nr : %3ld   max N : %8.2f kN  [ %5.2f m ]\n\r",
	  (*tmp_stab).l_nummer, M, pos * (*tmp_stab).d_laenge);

  fprintf(d,"\n\r");

  M = minMKette(staebe, &tmp_stab, &pos); 
  fprintf(d,"  Stab Nr : %3ld   min M : %8.2f kNm [ %5.2f m ]\n\r",
	  (*tmp_stab).l_nummer, M, pos * (*tmp_stab).d_laenge);

  M = minQKette(staebe, &tmp_stab, &pos);
  fprintf(d,"  Stab Nr : %3ld   min Q : %8.2f kN  [ %5.2f m ]\n\r",
	  (*tmp_stab).l_nummer, M, pos * (*tmp_stab).d_laenge);

  M = minNKette(staebe, &tmp_stab, &pos); 
  fprintf(d,"  Stab Nr : %3ld   min N : %8.2f kN  [ %5.2f m ]\n\r",
	  (*tmp_stab).l_nummer, M, pos * (*tmp_stab).d_laenge);

  fprintf(d,"\n\r");

  /* Ausgabe der Stbe */    
  liste_printStaebe(d, staebe, M_aktiv); 
     
  fprintf(d,"\f");

  /* Menge der aktiven Lastflle lschen */  
  Mng_MengeUndElementeLoeschen(M_aktiv); 

}



void liste_printStaebe(FILE* d, KeGlied* kette, Menge* M_aktiv)
{
  KeGlied *p,*p2;
  long i; 
  TypStab_Nm *tmp_stab;
  TypPunkt *tmp_punkt1,*tmp_punkt2; 
  TypMaterial *tmp_mat;
  TypQuerschnitt *tmp_quer; 
  double M,pos;
  TypStreckenlast_Nm *tmp_strecke;
  TypKnotenlast_Nm *tmp_punkt;
  char c_einheit[5];


  if(!kette)
    return;

  KeSchnellSortieren( kette, (Rel (*)(void*,void*)) Rel_Stab_Nm );


  KeDurchlaufVorwaerts(p, kette) {

    tmp_stab = p->x;

    fprintf(d,"  ----------------------------------------\n\n\r");
    fprintf(d,"   Nummer : %ld  \n\n\r",(*tmp_stab).l_nummer);
    fprintf(d,"  ----------------------------------------\n\r");
    tmp_punkt1 = (*tmp_stab).pe_punkta;
    tmp_punkt2 = (*tmp_stab).pe_punktb;
    fprintf(d,"\n\r");
    
    fprintf(d,"  Geometrie :\n\r");
    fprintf(d,"     Von  Punkt : [%2ld] (%.2f/%.2f) \n\r", 
	    (*tmp_punkt1).l_nummer, (*tmp_punkt1).d_x, (*tmp_punkt1).d_y);
    
    fprintf(d,"     Nach Punkt : [%2ld] (%.2f/%.2f)\n\r", 	     
	    (*tmp_punkt2).l_nummer, (*tmp_punkt2).d_x, (*tmp_punkt2).d_y );
    
    fprintf(d,"     Lnge : %8.3f m   Winkel : %8.3f rad \n\r", 
	    (*tmp_stab).d_laenge, (*tmp_stab).d_alpha);
    fprintf(d,"\n\r");
    

    tmp_quer = Suche_Querschnitt( (*tmp_stab).l_querid );

    if( tmp_quer ) {
      fprintf(d,"  Querschnitt : %s\n\r", (*tmp_quer).c_name);
      fprintf(d,"     Iz : %.6f m^4  A : %.4f m\n\r",
	      (*tmp_quer).d_Iz,(*tmp_quer).d_A);
    }
    else {
      fprintf(d,"  Querschnitt : kein Querschnitt !\n\r");
      fprintf(d,"     Iz : ---  m^4  A : --- m\n\r");
    }

    fprintf(d,"\n\r");
    

    tmp_mat = Suche_Material( (*tmp_stab).l_matid );

    if( tmp_mat ) {
      fprintf(d,"  Material :    %s\n\r", (*tmp_mat).c_name);
      fprintf(d,"     E : %.4e kN/m\n\r", (*tmp_mat).d_E);
    }
    else {
      fprintf(d,"  Material :    kein Material ! \n\r");
      fprintf(d,"     E : -----  kN/m\n\r");
    }

    fprintf(d,"\n\r");          
    


    /* Lasten des Stabes */
    fprintf(d,"  Lasten :    \n\r");      

    /* Streckenlasten durchsuchen */ 
    if(Modell->KeStreckenlast_Nm) {
      KeDurchlaufVorwaerts(p2,Modell->KeStreckenlast_Nm) {
	tmp_strecke = p2->x;
	
	/* aktuelle Streckenlast aktiv ? */
	if(Element_suchen( (*tmp_strecke).l_lastfallid ,M_aktiv)) 
	  if( (*tmp_strecke).l_stabnummer == (*tmp_stab).l_nummer ) {
	    
	    /* Ausgabe Streckenlast */
	    fprintf(d,"   [SL] LF %ld  %s\n\r",
		    suche_Lastfall_Nummer((*tmp_strecke).l_lastfallid),
		    (*tmp_strecke).c_name);
	    fprintf(d,"     A : %.3f kN/m  B : %.3f kN/m   Position : von %.2f m bis %.2f m\n\r",
		    (*tmp_strecke).d_lasta,
		    (*tmp_strecke).d_lastb,
		    (*tmp_strecke).d_anf * (*tmp_stab).d_laenge,
		    (*tmp_strecke).d_end * (*tmp_stab).d_laenge);
 
	  }
      }
    }
      
    /* Punktlasten durchsuchen */  
    if(Modell->KePunktlast_Nm) {
      KeDurchlaufVorwaerts(p2,Modell->KePunktlast_Nm) {
	tmp_punkt = p2->x;
	
	/* aktuelle Punktlast aktiv ? */
	if( Element_suchen((*tmp_punkt).l_lastfallid,M_aktiv))
	  if( (*tmp_punkt).l_stabnummer == (*tmp_stab).l_nummer ) {
	    fprintf(d,"   [PL] LF %ld  %s\n\r", 
		    suche_Lastfall_Nummer((*tmp_punkt).l_lastfallid), 
		    (*tmp_punkt).c_name);

	    if( (*tmp_punkt).l_art == 1 )
	      sprintf(c_einheit,"kN");
	    else
	      sprintf(c_einheit,"kNm");

	    fprintf(d,"    Gre : %.3f %s  Position %.2f\n\r",  
		    (*tmp_punkt).d_last,
		    c_einheit,
		    (*tmp_punkt).d_pos * (*tmp_stab).d_laenge);

	  }
      }
    }  

    fprintf(d,"\n\n\r"); 

    /* Schnittgren des Stabes */
    fprintf(d,"  Schnittgren :    \n\r");      

    /* Momente */
    fprintf(d,"  - Moment [kNm] - \n\r");
    
    fprintf(d,"    ");
    for(i=0;i<5;i++) 
      fprintf(d," %11.3f ",getM(tmp_stab,(double) (i/9.)));
    fprintf(d,"\n\r");
    
    fprintf(d,"    ");
    for(i=5;i<10;i++) 
      fprintf(d," %11.3f ",getM(tmp_stab,(double) (i/9.)));
    fprintf(d,"\n\n\r");
    
    
    /* Querkraft */
    fprintf(d,"  - Querkraft [kN] - \n\r");
    
    fprintf(d,"    ");
    for(i=0;i<5;i++) 
      fprintf(d," %11.3f ",getQ(tmp_stab,(double) (i/9.)));
    fprintf(d,"\n\r");
    
    fprintf(d,"    ");
    for(i=5;i<10;i++) 
      fprintf(d," %11.3f ",getQ(tmp_stab,(double) (i/9.)));
    fprintf(d,"\n\n\r");
    
    /* Normalkraft */
    fprintf(d,"  - Normalkraft [kN] - \n\r");
    
    fprintf(d,"    ");
    for(i=0;i<5;i++) 
      fprintf(d," %11.3f ",getN(tmp_stab,(double) (i/9.)));
    fprintf(d,"\n\r");
    
    fprintf(d,"    ");
    for(i=5;i<10;i++) 
      fprintf(d," %11.3f ",getN(tmp_stab,(double) (i/9.)));
    fprintf(d,"\n\n\r");
    
    /* Verschiebungen w */
    fprintf(d,"  - Verschiebungen senkrecht zur Stabachse [m] - \n\r");
    
    fprintf(d,"    ");
    for(i=0;i<5;i++) 
      fprintf(d," %11.5f ",get_w(tmp_stab,(double) (i/9.)));
    fprintf(d,"\n\r");
    
    fprintf(d,"    ");
    for(i=5;i<10;i++) 
      fprintf(d," %11.5f ",get_w(tmp_stab,(double) (i/9.)));
    fprintf(d,"\n\n\r");
    
    /* Verschiebungen u */
    fprintf(d,"  - Verschiebungen in Stabachse [m] - \n\r");
    
    fprintf(d,"    ");
    for(i=0;i<5;i++) 
      fprintf(d," %11.5f ",get_u(tmp_stab,(double) (i/9.)));
    fprintf(d,"\n\r");
    
    fprintf(d,"    ");
    for(i=5;i<10;i++) 
      fprintf(d," %11.5f ",get_u(tmp_stab,(double) (i/9.)));
    fprintf(d,"\n\n\r");
    
    
    /* Extrema */
    fprintf(d,"  - Extrema : \n\r");
    
    fprintf(d,"  ");
    M = maxM(tmp_stab,&pos);
    pos *= (*tmp_stab).d_laenge;
    fprintf(d,"  M  max = %11.3f kNm [ %4.2f m ]", M, pos);

    fprintf(d,"  ");
    M = minM(tmp_stab,&pos);
    pos *= (*tmp_stab).d_laenge;
    fprintf(d,"  min = %11.3f kNm [ %4.2f m ]\n\r", M, pos);

    fprintf(d,"  ");
    M = maxQ(tmp_stab,&pos);
    pos *= (*tmp_stab).d_laenge;
    fprintf(d,"  Q  max = %11.3f kN  [ %4.2f m ]", M, pos);

    fprintf(d,"  ");
    M = minQ(tmp_stab,&pos);
    pos *= (*tmp_stab).d_laenge;
    fprintf(d,"  min = %11.3f kN  [ %4.2f m ]\n\r", M, pos);

    fprintf(d,"  ");
    M = maxN(tmp_stab,&pos);
    pos *= (*tmp_stab).d_laenge;
    fprintf(d,"  N  max = %11.3f kN  [ %4.2f m ]", M, pos);

    fprintf(d,"  ");
    M = minN(tmp_stab,&pos);
    pos *= (*tmp_stab).d_laenge;
    fprintf(d,"  min = %11.3f kN  [ %4.2f m ]\n\r", M, pos);

    fprintf(d,"  ");
    M = max_w(tmp_stab,&pos);
    pos *= (*tmp_stab).d_laenge;
    fprintf(d,"  w  max = %11.5f m   [ %4.2f m ]", M, pos);

    fprintf(d,"  ");
    M = min_w(tmp_stab,&pos);
    pos *= (*tmp_stab).d_laenge;
    fprintf(d,"  min = %11.5f m   [ %4.2f m ]\n\r", M, pos);

    fprintf(d,"  ");
    M = max_u(tmp_stab,&pos);
    pos *= (*tmp_stab).d_laenge;
    fprintf(d,"  u  max = %11.5f m   [ %4.2f m ]", M, pos);

    fprintf(d,"  ");
    M = min_u(tmp_stab,&pos);
    pos *= (*tmp_stab).d_laenge;
    fprintf(d,"  min = %11.5f m   [ %4.2f m ]\n\r", M, pos);

    fprintf(d,"\n\n\r");
  }     

}


void liste_options()
{  
}

