/*
  Datei     : ps_zeichne_system.c
  erstellt  : 02.08.97
  geaendert : 
  Autor     : jh
*/


#include "XStabDek.h"

extern KeGlied* Staebe;


void ps_ZeichneSystem(TypMaszstab mass, double gray)
{
  double xa,ya;  /* Anfangspunkt */
  double xe,ye;  /* Endpunkt     */
  double breite;

  KeGlied *p;
  TypStab_Nm *tmp_stab;
  TypPunkt *tmp_punkt1,*tmp_punkt2;

  if(!Staebe)
    return;

  ps_linewidth(0.4);
  ps_linestyle(0); 

  KeDurchlaufVorwaerts(p,Staebe) {      
    tmp_stab = p->x;     
    tmp_punkt1 = (*tmp_stab).pe_punkta;
    tmp_punkt2 = (*tmp_stab).pe_punktb;
    xa = (*tmp_punkt1).d_x;
    ya = (*tmp_punkt1).d_y;
    xe = (*tmp_punkt2).d_x;
    ye = (*tmp_punkt2).d_y;

    ps_setgray(gray);
    ps_line(xa*mass.system + mass.off_x, 
	    ya*mass.system + mass.off_y,
	    xe*mass.system + mass.off_x, 
	    ye*mass.system + mass.off_y);

    /* Punkte zeichnen */
    ps_setgray(0.7);

    breite = 0.6;

    ps_rect((xa * mass.system + mass.off_x) - breite/2,
	    (ya * mass.system + mass.off_y) - breite/2,
	    breite, breite);

    ps_rect((xe * mass.system + mass.off_x) - breite/2,
	    (ye * mass.system + mass.off_y) - breite/2,
	    breite, breite);
   
    
  }    
}



void ps_ZeichneBezugsfaser(TypMaszstab mass)
{
  double xa,ya;
  double xe,ye;
  double abstand = 0.7;
  double s,c;

  KeGlied *p;
  TypStab_Nm *tmp_stab;
  TypPunkt *tmp_punkt1,*tmp_punkt2;

  if(!Staebe)
    return;

  ps_linewidth(0.1);
  ps_linestyle(1);
  

  KeDurchlaufVorwaerts(p,Staebe) {
    tmp_stab = p->x;     
    tmp_punkt1 = (*tmp_stab).pe_punkta;
    tmp_punkt2 = (*tmp_stab).pe_punktb;
    
    xa = (*tmp_punkt1).d_x;
    ya = (*tmp_punkt1).d_y;
    xe = (*tmp_punkt2).d_x;
    ye = (*tmp_punkt2).d_y;    
    
    s =  (*tmp_stab).d_sin;
    c =  (*tmp_stab).d_cos;
    
    ps_line(xa*mass.system + mass.off_x + (abstand * s), 
	    ya*mass.system + mass.off_y - (abstand * c),
	    xe*mass.system + mass.off_x + (abstand * s), 
	    ye*mass.system + mass.off_y - (abstand * c) );
    
  }    
}



void ps_ZeichneStabNummern(TypMaszstab mass)
{
  double xa,ya;
  double xb,yb;
  double x,y;

  KeGlied *p;
  TypPunkt *tmp_punkt1,*tmp_punkt2;
  TypStab_Nm *tmp_stab;
  char c_nummer[10];
 

  if(!Staebe)
    return;

  /* Zeichen - Einstellungen */
 
  ps_text_font("Times-Roman",12);


  KeDurchlaufVorwaerts(p,Staebe) {
    tmp_stab = p->x;     
      
    sprintf(c_nummer,"%ld", (*tmp_stab).l_nummer );

    tmp_punkt1 = (*tmp_stab).pe_punkta;
    tmp_punkt2 = (*tmp_stab).pe_punktb;
    
    xa = (*tmp_punkt1).d_x;
    ya = (*tmp_punkt1).d_y;
    xb = (*tmp_punkt2).d_x;
    yb = (*tmp_punkt2).d_y;

    x = 2 + (0.5 * (xa+xb)) * mass.system + mass.off_x;
    y = 2 + (0.5 * (ya+yb)) * mass.system + mass.off_y;

    ps_setgray(0);    
    ps_drawtext(x,y,c_nummer);

    ps_setgray(0.3);
    ps_linewidth(0.2);
    ps_rect( x-4, y-4, 10, 10);
   
  }

}


void ps_zeichne_FG(TypMaszstab mass)
{
  long i,von,bis;
  double xa,ya,xe,ye;
  double x1,x2,y1,y2;
  double alpha;

  KeGlied *p;
  TypStab_Nm *tmp_stab;
  TypPunkt *tmp_punkt;
  TypFreiGrad *tmp_freigrad;
  double abstand;
  double ab=.3;   /*  Abstand vom Endpunkt */

  double laenge=8; /* Lnge der Pfeile */

  char c_nummer[10];

  if(!Staebe)
    return;


  /* Zeichen - Einstellungen */

  ps_setgray(0.3);
  ps_linewidth(0.5);
  ps_text_font("Times-Roman",8);


  /* Schleife ber alle Stbe */
  KeDurchlaufVorwaerts(p,Staebe) {

    tmp_stab = p->x;     
    
    /* Koordinaten bestimmen */
    tmp_punkt = (*tmp_stab).pe_punkta;       
    xa = (*tmp_punkt).d_x;
    ya = (*tmp_punkt).d_y;    
    
    tmp_punkt = (*tmp_stab).pe_punktb;
    xe = (*tmp_punkt).d_x;
    ye = (*tmp_punkt).d_y;    
    
    abstand = ab;    
    
    /* Prfen, ob alle Freiheitsgrade gezeichnet werden mssen */   
    pruefeGelenke(&von, &bis, tmp_stab);
    
    /* Schleife ber die Freiheitsgrade eines Stabes */
    for(i=von; i<bis; i++) {

      tmp_freigrad = (*tmp_stab).pe_frei[i];
	
      if(i>2)
	abstand = 1.8 - ab;

      alpha =  (*tmp_freigrad).d_alpha;

      x1 = 0.5 * (xa * (2 - abstand) + xe * abstand);
      y1 = 0.5 * (ya * (2 - abstand) + ye * abstand); 	     

      switch ( (*tmp_freigrad).l_art) {	      

      case 0 :  /* X FG */
	      
	x1 = x1 * mass.system + mass.off_x;
	x2 = x1 + laenge * cos(alpha);
	      
	y1 = y1 * mass.system + mass.off_y;
	y2 = y1 - laenge * sin(alpha);

	ps_arrow(x1, y1, x2, y2);

	sprintf(c_nummer,"%ld", (*tmp_freigrad).l_nummer );     	     

	ps_drawtext(x2 + 1.3,
		    y2 + 0.3,
		    c_nummer);     
	break;
	      
      case 1 :	 /* Y FG */
	      
	x1 = x1 * mass.system + mass.off_x;
	x2 = x1 - laenge * sin(alpha);
	      
	y1 = y1 * mass.system + mass.off_y;
	y2 = y1 + laenge * cos(alpha);

	ps_arrow(x1, y1, x2, y2);

	sprintf(c_nummer,"%ld", (*tmp_freigrad).l_nummer );     	

	ps_drawtext(x2 + 1.3,
		    y2 + 1.3,
		    c_nummer); 
	break;
	
      case 2 :	      	  	
	   
	sprintf(c_nummer,"%ld",  (*tmp_freigrad).l_nummer );     

	ps_drawtext((x1 * mass.system + mass.off_x + laenge/3),
		    (y1 * mass.system + mass.off_y + laenge/3),
		    c_nummer);
	break;
	
      default :
	fprintf(stderr,"Fehler bei FreiGrad - Art !\n");
      }
    }            
  }    

}
