/*
  Datei     : Berechne.c
  erstellt  : 25.06.96
  geaendert : 10.11.97
  Autor     : jh
*/


#include "XStabDek.h"


void AufbauSysLast(double* ,long );
long AufbauPrimalDualStatus(double* ,double *,long *,long );
void AufbauProfil(long *Profil,long SysDim);
long AufbauSysSteif(double ***SysSteif,long *Profil,long SysDim);
void ErgebnisSpeichern(double *Primal,double *Dual);
void AufbauIndex(void);
void BerechneDual(double **, long *, double *, double *, double *, long);
long Fehler(void);
void MatrixMulti(double **erg, double **m1, double **m2,long dim);
long HoleEleSteif(double **elesteif,TypStab_Km *tmp_stab);
long Erstelle_C( double **C, TypStab_Km *tmp_stab);



/*********************************************************************/
/* void Berechne(void)                                               */
/*********************************************************************/ 
/*                                              
   Berechnen des Systems nach Theorie I. Ordnung
   jh, 13.04.97                          
*/
/*********************************************************************/      

void Berechne(void)
{
  long   *SysProfil = NULL;
  double **SysSteif = NULL;
  double *Primal    = NULL;
  double *Dual      = NULL;
  double *SysLast   = NULL;
  double *LastDual  = NULL;
  long *SysStatus   = NULL;
  long SysDim=0;
  KeGlied *p;
  long i,ERR;
  char c_text[200];
  double t1,t2,t3,t4;

  if(!Modell)
    return;

  /* DruckeFreiGrad(KeFreiGrad_Nm);  */

  t1 = (double) clock()/CLOCKS_PER_SEC;

  /* Nutzermodell ins Kernmodell transformieren */
  Nm_Trans_Km(); 

  Progress(0.1);

  Meldung("Kernmodell erstellt ...");
  XFlush(display);
    
  if(!Modell->KeFreiGrad_Km) {
    Meldung("Fehler beim Berechnen ! - keine FG");
    return;
  }

  if(!Modell->KeStab_Km) {
    Meldung("Fehler beim Berechnen ! - keine Stbe");
    return;
  }

  /* Systemdimension ermitteln */
  KeDurchlaufVorwaerts(p,Modell->KeFreiGrad_Km)
    SysDim++;

  sprintf(c_text,"Anzahl Freiheitsgrade : %ld ... ",SysDim);
  Meldung(c_text);
  XFlush(display);
  
  if(Fehler()) {
    if(!(Primal    = (double*)  calloc (SysDim,sizeof(double)))) fprintf(stderr,"Fehler !");
    if(!(Dual      = (double*)  calloc (SysDim,sizeof(double)))) fprintf(stderr,"Fehler !");
    if(!(SysLast   = (double*)  calloc (SysDim,sizeof(double)))) fprintf(stderr,"Fehler !");
    if(!(LastDual  = (double*)  calloc (SysDim,sizeof(double)))) fprintf(stderr,"Fehler !");
    if(!(SysStatus = (long*)    calloc (SysDim,sizeof(long))))   fprintf(stderr,"Fehler !"); 
    if(!(SysProfil = (long*)    calloc (SysDim,sizeof(long))))   fprintf(stderr,"Fehler !"); 
    
    /* fortlaufende Indices fr die FG vergeben */
    AufbauIndex();

    Progress(0.2);

    AufbauSysLast(SysLast,SysDim);
    
    Progress(0.3);

    if(!AufbauPrimalDualStatus(Primal,Dual,SysStatus,SysDim)) {     
      free(Primal);
      free(Dual);
      free(SysLast);
      free(LastDual);
      free(SysProfil);
      free(SysStatus);     
      return;
    }
      
    AufbauProfil(SysProfil,SysDim);            
          
    if(!AufbauSysSteif(&SysSteif,SysProfil,SysDim)) { 
      free(Primal);
      free(Dual);
      free(SysLast);
      free(LastDual);
      free(SysProfil);
      free(SysStatus);     
      return;
    }

    Meldung("Systemsteifigkeit aufgebaut ...");
    XFlush(display);          
    
    t3 = (double) clock()/CLOCKS_PER_SEC;

    Progress(0.4);

    Meldung("Zerlege Systemsteifigkeit ...");
    XFlush(display); 

    if( PmDecompose( SysDim, SysStatus, SysProfil, SysSteif,&ERR) == CPmPivot) {
      Meldung("Fehler beim Zerlegen der SystemSteifigkeit ! System kinematisch !");	
      free(Primal);
      free(Dual);
      free(SysLast);
      free(LastDual);
      free(SysProfil);
      free(SysStatus);    
      free(SysSteif[0]);
      free(SysSteif);     
      return;
    }

    Progress(0.7);

    t4 = (double) clock()/CLOCKS_PER_SEC;

    sprintf(c_text,"Gleichungssystem zerlegt [%.2f s] ... ",t4-t3);
    Meldung(c_text);   
    XFlush(display);
    
    for(i=0;i<SysDim;i++) {
      LastDual[i] = Dual[i] + SysLast[i]; 	  
    }  	
    
    PmSolveEqn( SysDim, SysStatus, SysProfil, SysSteif, Primal, LastDual); 

    Progress(0.9);

    Meldung("Gleichungssystem gelst ... ");
    XFlush(display);  
    
    /* unzerlegte SystemSteifigkeit herstellen */
    if(!AufbauSysSteif(&SysSteif,SysProfil,SysDim)) { 
      free(Primal);
      free(Dual);
      free(SysLast);
      free(LastDual);
      free(SysProfil);
      free(SysStatus);     
      return;
    }

    /* dualen Vektor fr Nachlaufrechnung berechnen, 
       der duale Vektor wird beim Gleichgungslser nicht bestimmt */
    BerechneDual(SysSteif, SysProfil, Primal, SysLast, Dual, SysDim); 
    
    ErgebnisSpeichern(Primal,Dual);       
    
    Nachlauf();       	

    Meldung("Nachlaufrechnung durchgefhrt ... ");
    XFlush(display);

    /* Ergebnisse des Kernmodells ins Nutzermodell eintragen */
    Km_Trans_Nm();

    t2 = (double) clock()/CLOCKS_PER_SEC;
   
    sprintf(c_text,"Rechenzeit : %.2f s",t2-t1);
    Meldung(c_text);
    Meldung("-> System nach Theorie I.Ordnung berechnet !"); 
    Zeichne();

    Progress(1.);

    XFlush(display);

    /* Speicher freigeben */   
    free(Primal);
    free(Dual);
    free(SysLast);
    free(LastDual);
    free(SysProfil);
    free(SysStatus);
    free(SysSteif[0]);
    free(SysSteif);

  }   

  Progress(0.);

}



/*********************************************************************/
/* void AufbauIndex(void)                                            */
/*********************************************************************/
/*                                              
   Funktion vergibt fortlaufende Indices 
   fr die Freiheitsgrade
   jh, 17.04.97 
*/
/*********************************************************************/

void AufbauIndex(void)
{
  long i=0;
  KeGlied *p;
  TypFreiGrad *tmp_freigrad; 

  KeDurchlaufVorwaerts(p, Modell->KeFreiGrad_Km) {
    tmp_freigrad = p->x;
    (*tmp_freigrad).l_index = i;
    i++;
  } 
  
}



/*********************************************************************/
/* long AufbauPrimalDualStatus( ... )                                */
/*********************************************************************/
/*                                              
   Funktion baut den Statusvektor, den primalen Vektor und
   den dualen Vektor auf
   jh, 17.04.97 
*/
/*********************************************************************/

long AufbauPrimalDualStatus(double *Primal, double *Dual,
			    long *SysStatus, long SysDim)
{
  long i;
  KeGlied *p;
  TypFreiGrad *tmp_freigrad;

  for(i=0;i<SysDim;i++)
    SysStatus[i] = CPmDual;
  
  KeDurchlaufVorwaerts(p,Modell->KeFreiGrad_Km)  {
    tmp_freigrad = p->x;

    if((*tmp_freigrad).l_index >= SysDim)  {
      Meldung("Fehler ! Index eines Freiheitsgrades groesser als SystemDimension ");		  
      return 0;
    }
      
    SysStatus[(*tmp_freigrad).l_index] = (*tmp_freigrad).l_status;
      
    switch ( SysStatus[(*tmp_freigrad).l_index] )  {
    case CPmPrimal :
      Primal[(*tmp_freigrad).l_index] = (*tmp_freigrad).d_verschiebung;
      break;
    case CPmDual :	  
      Dual[(*tmp_freigrad).l_index] = (*tmp_freigrad).d_einwirkung;
      break;
    default :
      Meldung("Fehler beim Aufbau Primal/Dual !");
      return 0;
    }
  }
  return 1;  
}



/*********************************************************************/
/* void AufbauProfil(long *Profil,long SysDim)                       */
/*********************************************************************/
/*                                              
   Aufbau des Profilvektors der Systemsteifigkeit,
   magebend ist der kleinste Index eines Elementes
   SysSteif -> sym. Profilmatrix
   jh, 13.04.96
   jh, 10.11.97, bug fixed : min = sysddim, danke lutz !
*/
/*********************************************************************/

void AufbauProfil(long *Profil, long SysDim)
{
  KeGlied *p;
  TypFreiGrad *tmp_freigrad;
  TypStab_Km *tmp_stab;
  long min,i;
  long f[6];
  
  for(i=0; i<SysDim; i++)
    Profil[i] = SysDim;

  /* Schleife ber alle Stbe */
  KeDurchlaufVorwaerts(p, Modell->KeStab_Km) {

    tmp_stab = p->x;      

    min = SysDim;

    for(i=0; i<6; i++) {

      tmp_freigrad = (*tmp_stab).pe_frei[i];

      f[i] = (*tmp_freigrad).l_index;	

      if ( (*tmp_freigrad).l_index < min ) 
	min = (*tmp_freigrad).l_index;

    }         

    for(i=0; i<6; i++)
      if ( Profil[ f[i] ] > min )
	Profil[ f[i] ] = min;      

  }
}
  


/*********************************************************************/
/* long AufbauSysSteif(double ***SysSteif,long *Profil,long SysDim)  */
/*********************************************************************/
/*                                              
   Aufbau der Systemsteifigkeit
   -> Einbau der Elementsteifigkeiten und Federsteifigkeiten  
   in die Systemsteifigkeit
   jh, 25.04.97                                 
*/
/*********************************************************************/

long AufbauSysSteif(double ***SysSteif,long *Profil,long SysDim)
{
  KeGlied *p;
  TypStab_Km *tmp_stab;
  char c_text[100];
  long i,j;
  TypFreiGrad *tmp_freigrad;
  long index[6];
  double **EleSteif;

  EleSteif = AllocMatrix(6,6);

  /* wenn Speicher schon geholt, dann initialisieren */
  if(*SysSteif) {
    for(i=0;i<SysDim;i++) {
      for(j=Profil[i]; j<=i; j++)
	(*SysSteif)[i][j] = 0.;
    }
  }
  else
    *SysSteif = AllocPMatrix(SysDim,Profil);
  
  if(!*SysSteif) {  
    Meldung("Speicherfehler !");
    return 0;
  }

  KeDurchlaufVorwaerts(p,Modell->KeStab_Km) {    
    tmp_stab = p->x;
    
    /* Aufbau des Indexvektors */        
    for(i=0;i<6;i++)  {
      tmp_freigrad = (*tmp_stab).pe_frei[i];
      index[i] = (*tmp_freigrad).l_index;    
      if( ((*tmp_freigrad).l_art) != i%3) {
	sprintf(c_text,"Fehler bei Freiheitsgrad %ld",i);
	Meldung(c_text);
	return 0;
      }
    }

    /* Elementsteifigkeit berechnen */    
    HoleEleSteif(EleSteif,tmp_stab);

    /* Einbau Elementsteifigkeit in Systemsteifigkeit */
    for(i=0;i<6;i++)
      for(j=0;j<6;j++) {
	if(index[i] >= index[j]) {
	  (*SysSteif)[ index[i] ][ index[j] ] += EleSteif[i][j];
	}	   
      }
  }

  /* Einbau der Federsteifigkeiten in die Systemsteifigkeit */
  KeDurchlaufVorwaerts(p,Modell->KeFreiGrad_Km) {
    tmp_freigrad = p->x;
    /* Federsteifigkeit auf Diagonalelement aufaddieren */
    (*SysSteif)[ (*tmp_freigrad).l_index ][  (*tmp_freigrad).l_index ] += (*tmp_freigrad).d_feder;   
  }
  
  free(EleSteif[0]);
  
  return 1;
}



/*********************************************************************/
/* long HoleEleSteif(double **elesteif,TypStab_Km *tmp_stab)         */
/*********************************************************************/
/*                                              
   Elementsteifigkeit nestimmen
   jh, 25.04.97                                 
*/
/*********************************************************************/

long HoleEleSteif(double **elesteif,TypStab_Km *tmp_stab)
{
  double **C;
  double **tmp;
  TypMaterial *mat;
  TypQuerschnitt *quer;
  double E,A,I;
  double l,l2,l3;
  long i,j;
  double s,c,alpha;

  C = AllocMatrix(6,6);
  tmp = AllocMatrix(6,6);

  for(i=0;i<6;i++)
    for(j=0;j<6;j++)  {	
      elesteif[i][j] = 0;
    }
  
  alpha = (*tmp_stab).d_alpha;
  c = cos( alpha );
  s = sin( alpha );
  l = (*tmp_stab).d_laenge;
  l2 = l*l;
  l3 = l*l*l;

  mat = Suche_Material( (*tmp_stab).l_matid );
  if(!mat)  {
    Meldung("es existieren Stbe ohne Material !");
    return 0;
  }

  E = (*mat).d_E;
     
  quer = Suche_Querschnitt( (*tmp_stab).l_querid );
  if(!quer)  {
    Meldung("es existieren Stbe ohne Querschnitt !");
    return 0;
  }

  A = (*quer).d_A;
  I = (*quer).d_Iz; 

  Erstelle_C(C,tmp_stab);

  /* Erstelle lokale Elementsteifigkeit */

  elesteif[0][0] = E*A/l;
  elesteif[0][3] = -E*A/l;

  elesteif[1][1] = 12*E*I/l3;
  elesteif[1][2] = 6*E*I/l2;

  elesteif[1][4] = -12*E*I/l3;
  elesteif[1][5] = 6*E*I/l2;
  
  elesteif[2][1] = 6*E*I/l2;
  elesteif[2][2] = 4*E*I/l;
  
  elesteif[2][4] = -6*E*I/l2;
  elesteif[2][5] = 2*E*I/l;


  elesteif[3][0] = -E*A/l;
  elesteif[3][3] = E*A/l;

  elesteif[4][1] = -12*E*I/l3;
  elesteif[4][2] = -6*E*I/l2;

  elesteif[4][4] = 12*E*I/l3;
  elesteif[4][5] = -6*E*I/l2;

  elesteif[5][1] = 6*E*I/l2;
  elesteif[5][2] = 2*E*I/l;

  elesteif[5][4] = -6*E*I/l2;
  elesteif[5][5] = 4*E*I/l;


  MatrixMulti(tmp,C,elesteif,6);


  /* Erstelle Ct  */

  C[0][1] *= -1;
  C[1][0] *= -1;
  C[3][4] *= -1;
  C[4][3] *= -1;

  MatrixMulti(elesteif,tmp,C,6);

  free(C[0]);
  free(tmp[0]);

  return 1;
}


long Erstelle_C( double **C, TypStab_Km *tmp_stab)
{  
  long i;
  TypFreiGrad *tmp_frei;
  double alpha;

  alpha = (*tmp_stab).d_alpha;
  
  for(i=0;i<6;i++)  {      
    tmp_frei = (*tmp_stab).pe_frei[i];

    if( i!=2 && i!=5)
      C[i][i] = cos( alpha - (*tmp_frei).d_alpha );      
    else
      C[i][i] = 1.;
      
    if( i==2 || i==5 )
      if( C[i][i] != 1)  {
	Meldung("Fehler : Drehung eines Z_Freiheitsgrades !");
	return 0;
      }      
      
    if( (i%3) == 0 )
      C[i][i+1] =  -sin( alpha - (*tmp_frei).d_alpha );  
      
    if( (i%3) == 1 )
      C[i][i-1] =  sin( alpha - (*tmp_frei).d_alpha );  
      
  } 
  return 1;
}



void ErgebnisSpeichern(double *Primal,double *Dual)
{
  TypFreiGrad *tmp_freigrad;
  KeGlied *p;
  long num;

  KeDurchlaufVorwaerts(p,Modell->KeFreiGrad_Km)  {    
    tmp_freigrad = p->x;

    num = (*tmp_freigrad).l_index;      
    (*tmp_freigrad).d_verschiebung = Primal[num];

    if( (*tmp_freigrad).l_status == CPmPrimal )
      (*tmp_freigrad).d_auflager = Dual[num];      
    else
      (*tmp_freigrad).d_einwirkung = Dual[num];

    if( (*tmp_freigrad).d_feder != 0. )
      (*tmp_freigrad).d_auflager = Dual[num];
    
  }
}

  
void AufbauSysLast(double *SysLast,long SysDim)
{
  long i,j;
  KeGlied *tmp;
  TypStab_Km *tmp_stab;
  TypFreiGrad *tmp_freigrad;
  long f[6];
  double l;
  double pa,pe,na,ne;
  double **C;
  double Last[6];

  C = AllocMatrix(6,6);;

  for (i=0;i<SysDim;i++)
    SysLast[i]=0;
    
  KeDurchlaufVorwaerts(tmp,Modell->KeStab_Km)   {
    tmp_stab = tmp->x;
    
    /* Indexvektor erstellen */    
    for(i=0;i<6;i++) {
      tmp_freigrad = (*tmp_stab).pe_frei[i];
      f[i] = (*tmp_freigrad).l_index;    
    }           
    
    l = (*tmp_stab).d_laenge;
    
    pa = (*tmp_stab).d_lastp_a;
    pe = (*tmp_stab).d_lastp_b;
    na = (*tmp_stab).d_lastn_a;
    ne = (*tmp_stab).d_lastn_b;

    Last[0] = l * (na/3 + ne/6);
    Last[1] = l * ((7*pa)/20 + (3*pe)/20);
    Last[2] = l*l * (pa/20 + pe/30);

    Last[3] = l * (na/6 + ne/3);
    Last[4] = l * ((3*pa)/20 + (7*pe)/20 );
    Last[5] = -l*l * (pa/30 + pe/20);

    Erstelle_C(C,tmp_stab);

    for(i=0;i<6;i++)
      for(j=0;j<6;j++) {
	SysLast[ f[i] ] += C[i][j] * Last[j];
      }           
  }
  
  free(C[0]);
  
}



/*********************************************************************/
/* void BerechneDual( ... )                                          */
/*********************************************************************/ 
/*                                              
   Funktion berechnet den dualen Vektor durch Multiplikation
   Dual = SysSteif * Primal
   jh, 04.05.97
*/
/*********************************************************************/ 

void BerechneDual(double **SysSteif, long *SysProfil, 
		  double *Primal,    double *SysLast, 
		  double *Dual,      long SysDim)
{
  long i,j;

  /* Multiplikation */
  for(i=0;i<SysDim;i++)  {
    Dual[i]=0;
    for(j=SysProfil[i];j<SysDim;j++)
      if (j<=i)
	Dual[i] += SysSteif[i][j] * Primal[j];
      else
	if(SysProfil[j]<=i)
	  Dual[i] += SysSteif[j][i] * Primal[j];
  }
  
  /* Systemlast abziehen */
  for(i=0;i<SysDim;i++)
    Dual[i] -= SysLast[i];

}


/* moegliche Fehler abpruefen  */
long Fehler(void)
{
  KeGlied *p;
  TypStab_Km *tmp_stab;
  TypMaterial *tmp_mat;
  TypQuerschnitt *tmp_quer;

  if(Modell->KeStab_Km)  {
    KeDurchlaufVorwaerts(p,Modell->KeStab_Km) {
      tmp_stab = p->x;
	  
      tmp_mat = Suche_Material( (*tmp_stab).l_matid );
      if(!tmp_mat)   {
	Meldung("es existieren Stbe ohne Material !");
	return 0;
      }	
	  
      if(!(*tmp_mat).d_E) {
	Meldung("es existiert Material mit E = 0 !");
	return 0;
      }
      
      tmp_quer = Suche_Querschnitt( (*tmp_stab).l_querid );
      if(!tmp_quer)  {
	Meldung("es existieren Stbe ohne Querschnitt !");
	return 0;
      }
      
      if(!(*tmp_quer).d_A)  {
	Meldung("es existieren Querschnitte mit A = 0 !");
	return 0;
      }
      
      if(!(*tmp_quer).d_Iz) {
	Meldung("es existieren Querschnitte mit Iz = 0 !");
	return 0;
      }
    }
  }
  else  {
    Meldung("keine Stbe im System !");
    return 0;
  }
  
  return 1;
  
}


void MatrixMulti(double **erg, double **m1, double **m2,long dim)
{
  long i,j,k;
  
  for(i=0;i<dim;i++)
    for(j=0;j<dim;j++)
      erg[i][j] = 0;

  for(i=0;i<dim;i++)
    for(j=0;j<dim;j++)
      for(k=0;k<dim;k++)
	erg[i][j] += m1[i][k] * m2[k][j];
}
