/*
  Datei     : transform.c
  erstellt  : 06.01.97
  gendert  : 13.04.97
  Autor     : jh
  Bemerkung :
*/


#include "XStabDek.h"


Rel Rel_Teilpunkt(double* a, double* b);
int KePosSuchen(double pos, KeGlied *q);
void Loesche_TeilPunkte(KeGlied *p);
void merke_pos_Punktlasten(KeGlied *Kette, Menge *M_aktiv,long);
void merke_pos_Streckenlasten(KeGlied *Kette, Menge *M_aktiv,long);
void PunktlastenUebernehmen(Menge *M_aktiv);
void StreckenlastenUebernehmen(Menge *M_aktiv);
void TeileStaebe_Km(void);



#define EPS_LAST 1e-4



/*********************************************************************/
/* void Nm_Trans_Km(void)                                            */
/*********************************************************************/ 
/*                                              
   Funktion transformiert Nutzermodell -> Kernmodell
   jh, 17.04.97 
*/
/*********************************************************************/

void Nm_Trans_Km(void)
{ 
  Menge *M_aktiv=NULL;               /* Menge d. akt. Lastfaelle    */
  KeGlied *p_stab=NULL;
  KeGlied *p1,*p2;
  KeGlied *KeTeilPunkte = NULL;      /* Teilpkte des akt. Stabes    */  
  TypStab_Nm *tmp_stab_Nm;           /* Akt. Stab im Nm             */
  TypStab_Km *tmp_stab_Km;
  double *tmp_pos;
  double posA,posB; 
  double xa,ya,xe,ye;
  double x,y;
  TypPunkt *tmp_punkt_Nm;
  TypPunkt *tmp_punkt_Km;
  TypFreiGrad **ZwFG;

  /* fprintf(stderr,"transformiere ...\n"); */

  if(!Modell->KeStab_Nm) {
    Meldung("kein System !");
    return;
  }

  /* altes Kernmodell lschen */
  Loesche_Km();

  /* Mengen und Ketten definieren */
  M_aktiv = Mng_MengeDefinieren();

  /* Bestimmen d. aktiven Lastflle */
  HoleAktiveLastfaelle_Lastwahl(M_aktiv); 

  /* Ketten definieren */  
  KeTeilPunkte = KeKetteDefinieren();  
 
  /* Schleife ber alle Stbe */                           
  KeDurchlaufVorwaerts(p_stab,Modell->KeStab_Nm) {
    long anz,k,j;
    long count;
    TypFreiGrad *tmp_fg_Nm;
    TypFreiGrad *tmp_fg_Km;

    tmp_stab_Nm = p_stab->x;
    
    /* Unstetigkeitsstellen ermitteln */          
    
    /* Anfang und Ende eintragen */
    tmp_pos = (double*) calloc (1,sizeof(double));
    *tmp_pos = 0.;
    KeElementEintragen(tmp_pos, Nach, KeTeilPunkte);
    
    tmp_pos = (double*) calloc (1,sizeof(double));
    *tmp_pos = 1.;
    KeElementEintragen(tmp_pos, Nach, KeTeilPunkte);
    
    /* Unstetigkeiten durch Strecken- und Punktlasten */
    merke_pos_Punktlasten(KeTeilPunkte, M_aktiv, (*tmp_stab_Nm).l_nummer); 
    merke_pos_Streckenlasten(KeTeilPunkte, M_aktiv, (*tmp_stab_Nm).l_nummer);
    
    /* Sortieren der Teilpunkte nach Position im Stab in
       aufsteigender Reihenfolge */
    KeSchnellSortieren(KeTeilPunkte, (Rel (*)(void*,void*)) Rel_Teilpunkt );
    
    /* Koordinaten des Anfangs- und Endpunktes holen */
    tmp_punkt_Nm = (*tmp_stab_Nm).pe_punkta;
    xa = (*tmp_punkt_Nm).d_x;   ya = (*tmp_punkt_Nm).d_y;

    tmp_punkt_Nm = (*tmp_stab_Nm).pe_punktb;
    xe = (*tmp_punkt_Nm).d_x;   ye = (*tmp_punkt_Nm).d_y;

    /* Anzahl Teil-Punkte ermitteln */
    anz=0;
    KeDurchlaufVorwaerts(p1,KeTeilPunkte) { 
      anz++;
    }

    /* Speicher fr die Freiheitsgrade der 
       Zwischenpunkte holen */
    ZwFG = (TypFreiGrad**) calloc ((anz-2)*3,sizeof(TypFreiGrad*)); 

    /* Freiheitsgrade fr ZwischenPunkte erzeugen */
    for(k=0;k<anz-2;k++) {
      for(j=0;j<3;j++) {
	tmp_fg_Km = Erzeuge_FG_Km(j,-1L);
	ZwFG[(k*3)+j] = tmp_fg_Km;
      }
    }
    
    /* Stbe im Kernmodell erzeugen */  
    count=0;
    /* Durchlauf der Teilpunkte von 0.0 nach 1.0 */
    KeDurchlaufVorwaerts(p1,KeTeilPunkte) {
      tmp_pos = p1->x;
      posA = (*tmp_pos);
      p2 = p1->nach;
      if(p2==KeTeilPunkte) break;
      tmp_pos = p2->x;
      posB = (*tmp_pos);

      /* fprintf(stderr,"Erzeuge Stab A[%.2f] B[%.2f]\n",posA,posB); */

      tmp_stab_Km = Erzeuge_Stab_Km((*tmp_stab_Nm).l_nummer);

      (*tmp_stab_Km).d_posA = posA;
      (*tmp_stab_Km).d_posB = posB;
      (*tmp_stab_Km).l_matid = (*tmp_stab_Nm).l_matid;
      (*tmp_stab_Km).l_querid = (*tmp_stab_Nm).l_querid;

      /* Punkte erzeugen */     
      /* Interpolation des Anfangspunktes */
      x = ((1.-posA) * xa) + (posA * xe);
      y = ((1.-posA) * ya) + (posA * ye);

      /* Erzeuge Anfangspunkt */      
      tmp_punkt_Km = Erzeuge_Punkt_Km(x,y);
      (*tmp_stab_Km).pe_punkta = tmp_punkt_Km;     

      /* Interpolation des Endunktes */
      x = ((1.-posB) * xa) + (posB * xe);
      y = ((1.-posB) * ya) + (posB * ye);

      /* Erzeuge Endpunkt */     
      tmp_punkt_Km = Erzeuge_Punkt_Km(x,y);
      (*tmp_stab_Km).pe_punktb = tmp_punkt_Km;     

      SetzeGeometrie_Km(tmp_stab_Km);          

      /* Erzeuge und setze Freiheitsgrade */
      if(posA==0.) { /* Anfang des Stabes -> FG bernehmen */
	for(j=0;j<3;j++) {
	  /* Hole FG aus Nm */
	  tmp_fg_Nm = (*tmp_stab_Nm).pe_frei[j];

	  /* Erzeuge FG im Km */
	  tmp_fg_Km = Erzeuge_FG_Km((*tmp_fg_Nm).l_art,(*tmp_fg_Nm).l_nummer);

	  /* Werte aus Nm kopieren */
	  (*tmp_fg_Km).d_verschiebung = (*tmp_fg_Nm).d_verschiebung;
	  (*tmp_fg_Km).d_einwirkung   = (*tmp_fg_Nm).d_einwirkung;	 
	  (*tmp_fg_Km).d_feder        = (*tmp_fg_Nm).d_feder;
	  (*tmp_fg_Km).l_status       = (*tmp_fg_Nm).l_status;	 	
	  (*tmp_fg_Km).d_alpha        = (*tmp_fg_Nm).d_alpha;

	  (*tmp_stab_Km).pe_frei[j] = tmp_fg_Km;

	}
      }
      else {
	for(j=0;j<3;j++) {	 	 	  
	  /* bernehme FG fr Zwischenpunkte */
	  tmp_fg_Km = ZwFG[(count*3)+j];

	  /* setze Werte */	  
	  (*tmp_fg_Km).d_feder  = 0;	  
	  (*tmp_fg_Km).l_status = CPmDual;
	  (*tmp_fg_Km).d_alpha  = 0;

	  (*tmp_stab_Km).pe_frei[j] = tmp_fg_Km;
		
	} 	
	count++;
      }
      
      if(posB==1.) {  /* Ende des Stabes -> FG bernehmen */
	for(j=0;j<3;j++) {

	  /* Hole FG aus Nm */
	  tmp_fg_Nm = (*tmp_stab_Nm).pe_frei[j+3];

	  /* Erzeuge FG im Km */
	  tmp_fg_Km = Erzeuge_FG_Km((*tmp_fg_Nm).l_art,(*tmp_fg_Nm).l_nummer);

	  /* Werte aus Nm kopieren */
	  (*tmp_fg_Km).d_verschiebung = (*tmp_fg_Nm).d_verschiebung;
	  (*tmp_fg_Km).d_einwirkung   = (*tmp_fg_Nm).d_einwirkung;	 
	  (*tmp_fg_Km).d_feder        = (*tmp_fg_Nm).d_feder;
	  (*tmp_fg_Km).l_status       = (*tmp_fg_Nm).l_status;	 	
	  (*tmp_fg_Km).d_alpha        = (*tmp_fg_Nm).d_alpha;

	  (*tmp_stab_Km).pe_frei[j+3] = tmp_fg_Km;
	
	}
      }
      else {
	for(j=0;j<3;j++) {	 	 
	  /* bernehme FG fr Zwischenpunkte */
	  tmp_fg_Km = ZwFG[(count*3)+j];	 

	  /* setze Werte */	 
	  (*tmp_fg_Km).d_feder  = 0;	  
	  (*tmp_fg_Km).l_status = CPmDual;
	  (*tmp_fg_Km).d_alpha  = 0;

	  (*tmp_stab_Km).pe_frei[j+3] = tmp_fg_Km;
	  
	}
      }      
    }      
    
    /* Teilpunkte des Stabes lschen */   
    Loesche_TeilPunkte(KeTeilPunkte);     
    
  }   

  /* ermitteln der Punktlasten, die auf den Stab wirken */
  PunktlastenUebernehmen(M_aktiv);

  /* ermitteln der Streckenlasten, die auf den Stab wirken */
  StreckenlastenUebernehmen(M_aktiv);

  /* Menge der aktiven Lastflle lschen */  
  Mng_MengeUndElementeLoeschen(M_aktiv); 

  /* Stbe teilen */
  TeileStaebe_Km();

  /* DruckeStaebe_Km(); */
  /* DruckePunkte_Km(); */

  /*fprintf(stderr,"+++\n"); 
  DruckeFreiGrad(KeFreiGrad_Km);  */

  /* fprintf(stderr,"Nutzermodell transformiert\n");      */
  
}



/*********************************************************************/
/* void PunktlastenUebernehmen( ... )                                */
/*********************************************************************/ 
/*                                              
   Funktion holt aus dem Nutzermodell die Punktlasten und schreibt
   sie in die Freiheitsgrade des Kernmodells   
   jh, 16.04.97    
*/
/*********************************************************************/

void PunktlastenUebernehmen(Menge *M_aktiv)
{
  KeGlied *p_pkt, *p_stab;
  TypFreiGrad *tmp_fg_Km1, *tmp_fg_Km2;
  TypKnotenlast_Nm *tmp_punktlast_Nm;
  TypStab_Km *tmp_stab_Km;
  TypLastfall *tmp_lastfall=NULL;
  double alpha;
  double phi;
  double gewicht;

  
  if(!Modell->KePunktlast_Nm)
    return;

  if(!Modell->KeStab_Km)
    return;

  /* Schleife ueber alle Punktlasten */
  KeDurchlaufVorwaerts(p_pkt,Modell->KePunktlast_Nm) {
    tmp_punktlast_Nm = p_pkt->x;
    
    /* aktuelle Punktlast aktiv ? */
    if( Element_suchen((*tmp_punktlast_Nm).l_lastfallid,M_aktiv) == 0 )
      continue;

    /* Gewichtung bestimmen */
    tmp_lastfall = Suche_Lastfall_id((*tmp_punktlast_Nm).l_lastfallid);
    if(!tmp_lastfall) {      
      Meldung("Fehler : PL kein Lastfall\n");      
      return;
    }
    gewicht = (*tmp_lastfall).d_gewicht;
    
    /* Stab im Kernmodell suchen */
    KeDurchlaufVorwaerts(p_stab, Modell->KeStab_Km) {
      tmp_stab_Km = p_stab->x;

      /* stimmen der Stab im Km und die Stabnummer der Last berein ? */
      if( (*tmp_punktlast_Nm).l_stabnummer != (*tmp_stab_Km).l_nummer )
	continue;  

      /* Last greift an Punkt A des Stabes im Km an */
      if( fabs((*tmp_stab_Km).d_posA - (*tmp_punktlast_Nm).d_pos) < EPS_LAST ) {

	/* Typ der Punktlast */
	switch( (*tmp_punktlast_Nm).l_art ) {

	case 1 :  /* x,y */
	  tmp_fg_Km1 = (*tmp_stab_Km).pe_frei[0];
	  tmp_fg_Km2 = (*tmp_stab_Km).pe_frei[1];
	  
	  /* Winkel berechnen */
	  if( (*tmp_punktlast_Nm).l_koordsys_winkel == 1)  /* lokal */
	    phi = (*tmp_punktlast_Nm).d_alpha + (*tmp_stab_Km).d_alpha;  
	  else /* global */
	    phi = (*tmp_punktlast_Nm).d_alpha;
	  
	  alpha = phi - (*tmp_fg_Km1).d_alpha;
	  
	  /* fprintf(stderr,"-A- Punktlast alpha[%.2f] Gre [%.2f]\n",
	     alpha, (*tmp_punktlast_Nm).d_last ); */

	  /* FG [0] */
	  (*tmp_fg_Km1).d_einwirkung += ((*tmp_punktlast_Nm).d_last * sin(alpha) * gewicht);
	  
	  /* FG [1] */
	  (*tmp_fg_Km2).d_einwirkung -= ((*tmp_punktlast_Nm).d_last * cos(alpha) * gewicht);

	  break;

	case 2 :  /* z */
	  tmp_fg_Km1 = (*tmp_stab_Km).pe_frei[2];

	  /* Aufaddieren der Einwirkung */
	  (*tmp_fg_Km1).d_einwirkung += ((*tmp_punktlast_Nm).d_last * gewicht);
	  break;

	default :
	  fprintf(stderr,
		  "Fehler (PunktlastenUebernehmen() : falscher Typ einer Punktlast !\n");
	  exit(1);
	}
	  
	break;
      }

      /* Last greift an Punkt B des Stabes im Km an */
      if( fabs((*tmp_stab_Km).d_posB - (*tmp_punktlast_Nm).d_pos) < EPS_LAST ) {
	
	/* Typ der Punktlast */
	switch( (*tmp_punktlast_Nm).l_art ) {
	  
	case 1 :  /* x,y */
	  tmp_fg_Km1 = (*tmp_stab_Km).pe_frei[3];
	  tmp_fg_Km2 = (*tmp_stab_Km).pe_frei[4];
	  
	  /* Winkel berechnen */
	  if( (*tmp_punktlast_Nm).l_koordsys_winkel == 1)  /* lokal */
	    phi = (*tmp_punktlast_Nm).d_alpha + (*tmp_stab_Km).d_alpha;  
	  else /* global */
	    phi = (*tmp_punktlast_Nm).d_alpha;
	  
	  alpha = phi - (*tmp_fg_Km1).d_alpha;	

	  /* FG [0] */
	  (*tmp_fg_Km1).d_einwirkung += ((*tmp_punktlast_Nm).d_last * sin(alpha) * gewicht);

	  /* FG [1] */
	  (*tmp_fg_Km2).d_einwirkung -= ((*tmp_punktlast_Nm).d_last * cos(alpha) * gewicht);

	  break;

	case 2 :  /* z */
	  tmp_fg_Km1 = (*tmp_stab_Km).pe_frei[5];

	  /* Aufaddieren der Einwirkung */
	  (*tmp_fg_Km1).d_einwirkung += ((*tmp_punktlast_Nm).d_last * gewicht);
	  break;

	default :
	  fprintf(stderr,
		  "Fehler (PunktlastenUebernehmen() : falscher Typ einer Punktlast !\n");
	  exit(1);
	}
	break;
      }
    }
    
  }
}



/*********************************************************************/
/* void StreckenlastenUebernehmen( ... )                             */
/*********************************************************************/ 
/*                                              
   Funktion holt aus dem Nutzermodell die Streckenlasten und schreibt
   sie in die Stbe des Kernmodells   
   jh, 16.04.97    
*/
/*********************************************************************/

void StreckenlastenUebernehmen(Menge *M_aktiv)
{
  KeGlied *p_str = NULL,*p_stab=NULL;  
  TypStreckenlast_Nm*  tmp_streckenlast_Nm  ;   
  TypStab_Km *tmp_stab_Km;
  TypLastfall *tmp_lastfall=NULL;
  double lastA,lastB;
  double alpha,phi;
  double last_a,last_b;
  double za,ze;
  double gewicht;
  
  if(!Modell->KeStreckenlast_Nm)
    return;

  if(!Modell->KeStab_Km)
    return;

  /* Schleife ber alle Stbe des Kernmodells */
  KeDurchlaufVorwaerts(p_stab, Modell->KeStab_Km) {
    tmp_stab_Km = p_stab->x;

    alpha = (*tmp_stab_Km).d_alpha;

    /* Schleife ber alle Streckenlasten */       
    KeDurchlaufVorwaerts(p_str,Modell->KeStreckenlast_Nm) {
      tmp_streckenlast_Nm = p_str->x;
    
      /* wenn aktuelle Streckenlast aktiv */
      if( Element_suchen((*tmp_streckenlast_Nm).l_lastfallid,M_aktiv) == 0 )
	continue;

      /* Gewichtung bestimmen */
      tmp_lastfall = Suche_Lastfall_id((*tmp_streckenlast_Nm).l_lastfallid);
      if(!tmp_lastfall)	{
	Meldung("Fehler : SL kein Lastfall\n");	 
	return;
      }
      gewicht = (*tmp_lastfall).d_gewicht;
      
      /* und Staebe bereinstimmen    */
      if( (*tmp_streckenlast_Nm).l_stabnummer != (*tmp_stab_Km).l_nummer )
	continue;
   
      /* liegt der Km-Stab im Bereich der Last ? */
      if( (*tmp_streckenlast_Nm).d_anf < (*tmp_stab_Km).d_posB )
	if( (*tmp_streckenlast_Nm).d_end > (*tmp_stab_Km).d_posA ) {

	  /* Sttzstellen im Bereich der Last ermitteln */
	  za = ((*tmp_stab_Km).d_posA - (*tmp_streckenlast_Nm).d_anf)
	    / ((*tmp_streckenlast_Nm).d_end - (*tmp_streckenlast_Nm).d_anf);

	  ze = ((*tmp_stab_Km).d_posB - (*tmp_streckenlast_Nm).d_anf)
	    / ((*tmp_streckenlast_Nm).d_end - (*tmp_streckenlast_Nm).d_anf);

	  /* Winkel berechnen */
	  if( (*tmp_streckenlast_Nm).l_koordsys == 1)  /* lokal */
	    phi = (*tmp_streckenlast_Nm).d_alpha;	  
	  else 
	    phi = (*tmp_streckenlast_Nm).d_alpha - alpha;	    

	  /* Last in last_p transformieren */
	  lastA = -(*tmp_streckenlast_Nm).d_lasta * cos(phi);
	  lastB = -(*tmp_streckenlast_Nm).d_lastb * cos(phi);
	  
	  /* interpolieren */
	  last_a = lastA * (1.-za) + lastB * za;
	  last_b = lastA * (1.-ze) + lastB * ze;

	  /* Last p auf Stab addieren */
	  (*tmp_stab_Km).d_lastp_a += (last_a * gewicht);
	  (*tmp_stab_Km).d_lastp_b += (last_b * gewicht);

	  /* Last in last_n transformieren */
	  lastA = (*tmp_streckenlast_Nm).d_lasta * sin(phi);
	  lastB = (*tmp_streckenlast_Nm).d_lastb * sin(phi);

	  /* interpolieren */
	  last_a = lastA * (1.-za) + lastB * za;
	  last_b = lastA * (1.-ze) + lastB * ze;

	  /* Last n auf Stab addieren */
	  (*tmp_stab_Km).d_lastn_a += (last_a * gewicht);
	  (*tmp_stab_Km).d_lastn_b += (last_b * gewicht);

	}      
    }
  }
}




/*********************************************************************/
/* void merke_Punktlasten( ... )                                     */
/*********************************************************************/ 
/*                                              
   Funktion vermerkt die Unstetigkeitsstellen in einem Stab
   resultierent aus Punktlasten
   jh, 13.04.97    
*/
/*********************************************************************/

void merke_pos_Punktlasten(KeGlied *Kette, Menge *M_aktiv, long stabnummer)
{
  KeGlied *p_pkt = NULL;
  TypKnotenlast_Nm* tmp_punktlast_Nm;     
  double *tmp_pos;

  if(!Modell->KePunktlast_Nm)
    return;

  /* Schleife ueber alle Punktlasten */
  KeDurchlaufVorwaerts(p_pkt,Modell->KePunktlast_Nm) {
    tmp_punktlast_Nm = p_pkt->x;
    
    /* aktuelle Punktlast aktiv ? */
    if( Element_suchen((*tmp_punktlast_Nm).l_lastfallid,M_aktiv) == 0 )
      continue;
    
    /* stimmen Stbe ueberein ? */
    if( (*tmp_punktlast_Nm).l_stabnummer != stabnummer )
      continue;	    	    
    
    /* Stelle merken */	  	    	   	  
    
    /* wenn Pkt.last nicht an Anf.- oder Endknoten angreift */
    if(((*tmp_punktlast_Nm).d_pos != 0.) && 
       ((*tmp_punktlast_Nm).d_pos != 1.)) {
      /* Wenn Punkt nicht schon vorhanden */
      if(!KePosSuchen((*tmp_punktlast_Nm).d_pos, Kette)) {
	/* Teilpunkt d. Stabes vermerken */		      
	tmp_pos = (double*) calloc (1,sizeof(double));
	*tmp_pos = (*tmp_punktlast_Nm).d_pos;
	KeElementEintragen(tmp_pos, Nach, Kette);
      }		
    }
  }
}



/*********************************************************************/
/* void merke_Streckenlasten( ... )                                  */
/*********************************************************************/ 
/*                                              
   Funktion vermerkt die Unstetigkeitsstellen in einem Stab
   resultierent aus Streckenlasten
   jh, 13.04.97
*/
/*********************************************************************/

void merke_pos_Streckenlasten(KeGlied *Kette, Menge *M_aktiv, long stabnummer)
{
  KeGlied *p_str = NULL;
  TypStreckenlast_Nm*  tmp_streckenlast_Nm  ; 
  double *tmp_pos;

  if(!Modell->KeStreckenlast_Nm)
    return;

  /* Schleife ber alle Streckenlasten */       
  KeDurchlaufVorwaerts(p_str,Modell->KeStreckenlast_Nm) {
    tmp_streckenlast_Nm = p_str->x;
    
    /* wenn aktuelle Streckenlast aktiv */
    if( Element_suchen((*tmp_streckenlast_Nm).l_lastfallid,M_aktiv) == 0 )
      continue;
    
    /* und Staebe uebereinstimmen    */
    if( (*tmp_streckenlast_Nm).l_stabnummer != stabnummer )
      continue;
    
    /* Stelle merken */	           		
    
    /* wenn Streckenlast nicht an Anfangsknoten des Stabes beginnt */
    if((*tmp_streckenlast_Nm).d_anf != 0.) {
      /* Wenn Punkt nicht schon vorhanden */
      if( !KePosSuchen((*tmp_streckenlast_Nm).d_anf,Kette) ) {	      
	/* Teilpunkt d. Stabes vermerken */
	tmp_pos = (double*) calloc (1,sizeof(double));
	*tmp_pos = (*tmp_streckenlast_Nm).d_anf;
	KeElementEintragen(tmp_pos, Nach, Kette);	
      }
    }
    
    /* wenn Streckenlast nicht an Endknoten des Stabs endet */
    if((*tmp_streckenlast_Nm).d_end != 1.) {
      /* Wenn Punkt nicht schon vorhanden */
      if( !KePosSuchen((*tmp_streckenlast_Nm).d_end, Kette) ) {
	/* Teilpunkt d. Stabes vermerken */
	tmp_pos = (double*) calloc (1,sizeof(double));
	*tmp_pos = (*tmp_streckenlast_Nm).d_end;  
	KeElementEintragen(tmp_pos,Nach,Kette);
      }
    }
  } 
}



/*********************************************************************/
/* void Loesche_TeilPunkte(KeGlied *p)                               */
/*********************************************************************/ 
/*                                              
   Funktion lscht die Kette mit den temp. Teilpunkten
   jh, 13.04.97
*/
/*********************************************************************/

void Loesche_TeilPunkte(KeGlied *p)
{
  KeGlied *l;

  if(p==NULL)
    return;

  while(p->nach != p) {	  
    l = p->nach;
    KeGliedAustragen(l);       
    free(l->x);
  }
}



/*********************************************************************/
/* void Km_Trans_Nm(void)                                            */
/*********************************************************************/ 
/*                                              
   Funktion transformiert die Ergebisse de Berechnung
   aus dem Kernmodell ins Nutzermodell 
   jh, 17.04.97 
*/
/*********************************************************************/

void Km_Trans_Nm(void)
{

  KeGlied *p,*l;
  KeGlied *tmp_intervalle;
  TypStab_Nm *tmp_stab_Nm;
  TypStab_Km *tmp_stab_Km;
  TypIntervall *tmp_intervall;
  TypPunkt *tmp_punkt_A, *tmp_punkt_B;
  TypFreiGrad *tmp_fg;
  TypFreiGrad *tmp_fg_Km, *tmp_fg_Nm;
  long i;

  
  if(!Modell->KeStab_Nm)
    return;

  /* Intervalle im Nm lschen */ 
  KeDurchlaufVorwaerts(p,Modell->KeStab_Nm) {
    tmp_stab_Nm = p->x; 
    
    tmp_intervalle = (*tmp_stab_Nm).intervalle;
    
    if(tmp_intervalle) {
      while(tmp_intervalle->nach != tmp_intervalle)  {	  
	l = tmp_intervalle->nach;
	KeGliedAustragen(l);       
	free(l->x);
      }
    }
  }

  /* Schleife ber alle Stbe im Km */
  KeDurchlaufVorwaerts(p,Modell->KeStab_Km) {
    tmp_stab_Km = p->x; 

    /* hole zugehrigen Stab im Nm */
    tmp_stab_Nm = Suche_Stab_Nm( (*tmp_stab_Km).l_nummer );

    /* wenn Stab im Km am Anfang des Stabes im Nm liegt
       -> Werte der FG bernehmen */
    if((*tmp_stab_Km).d_posA == 0.) {
      for(i=0;i<3;i++) {
	tmp_fg_Km = (*tmp_stab_Km).pe_frei[i];
	tmp_fg_Nm = (*tmp_stab_Nm).pe_frei[i];
	(*tmp_fg_Nm).d_auflager = (*tmp_fg_Km).d_auflager;
	(*tmp_fg_Nm).d_verschiebung = (*tmp_fg_Km).d_verschiebung;
      }
    }
	
    /* wenn Stab im Km am Ende des Stabes im Nm liegt
       -> Werte der FG bernehmen */
    if((*tmp_stab_Km).d_posB == 1.) {
      for(i=3;i<6;i++) {
	tmp_fg_Km = (*tmp_stab_Km).pe_frei[i];
	tmp_fg_Nm = (*tmp_stab_Nm).pe_frei[i];
	(*tmp_fg_Nm).d_auflager = (*tmp_fg_Km).d_auflager;
	(*tmp_fg_Nm).d_verschiebung = (*tmp_fg_Km).d_verschiebung;
      }
    }

    /* Werte des Stabes_Km in den Stab_Nm eintragen */
    tmp_intervall = Erzeuge_Intervall(tmp_stab_Nm);

    (*tmp_intervall).d_Na = (*tmp_stab_Km).d_Na;
    (*tmp_intervall).d_Nb = (*tmp_stab_Km).d_Nb;
    (*tmp_intervall).d_Qa = (*tmp_stab_Km).d_Qa;
    (*tmp_intervall).d_Qb = (*tmp_stab_Km).d_Qb;
    (*tmp_intervall).d_Ma = (*tmp_stab_Km).d_Ma;
    (*tmp_intervall).d_Mb = (*tmp_stab_Km).d_Mb;

    (*tmp_intervall).d_lastp_a = (*tmp_stab_Km).d_lastp_a;
    (*tmp_intervall).d_lastp_b = (*tmp_stab_Km).d_lastp_b;
    (*tmp_intervall).d_lastn_a = (*tmp_stab_Km).d_lastn_a;
    (*tmp_intervall).d_lastn_b = (*tmp_stab_Km).d_lastn_b;

    (*tmp_intervall).d_laenge = (*tmp_stab_Km).d_laenge;
    (*tmp_intervall).d_alpha_stab = (*tmp_stab_Km).d_alpha;

    tmp_punkt_A = (*tmp_stab_Km).pe_punkta;
    tmp_punkt_B = (*tmp_stab_Km).pe_punktb;

    (*tmp_intervall).d_xa = (*tmp_punkt_A).d_x;
    (*tmp_intervall).d_ya = (*tmp_punkt_A).d_y;
    (*tmp_intervall).d_xe = (*tmp_punkt_B).d_x;
    (*tmp_intervall).d_ye = (*tmp_punkt_B).d_y;

    (*tmp_intervall).d_posA = (*tmp_stab_Km).d_posA;
    (*tmp_intervall).d_posB = (*tmp_stab_Km).d_posB;

    for(i=0;i<6;i++) {
      tmp_fg = (*tmp_stab_Km).pe_frei[i];
      (*tmp_intervall).d_vs[i] = (*tmp_fg).d_verschiebung;     
    }

    tmp_fg = (*tmp_stab_Km).pe_frei[0];
    (*tmp_intervall).d_alpha1 = (*tmp_fg).d_alpha;
    tmp_fg = (*tmp_stab_Km).pe_frei[3];
    (*tmp_intervall).d_alpha2 = (*tmp_fg).d_alpha;

  }

}



/***********************************************************/
/* int KePosSuchen(double x, KeGlied *q)                   */
/***********************************************************/
/*
   Fkt sucht Position pos in der Kette q
   wenn vorhanden return = 1
   wenn nicht     return = 0
   as, 02.04.97
   jh, 13.04.97
*/
/***********************************************************/

int KePosSuchen(double pos, KeGlied *q)
{
  KeGlied* p;
  double* tmp_element;
  
  KeDurchlaufVorwaerts(p,q)  {
    tmp_element = p->x;
    if(fabs(pos - *tmp_element) < EPS_LAST)      /* wenn Element gefunden */
      return 1;
  }
  return 0;
}



/*********************************************************************/
/* Rel Rel_Teilpunkt(double *a, double *b)                           */
/*********************************************************************/ 
/*                                              
   Vergleichsfunktion fr Punkte
   Punkte werden nach ihrer Nummer sortiert
   as, 01.04.97                                 
   jh, 16.04.97
*/
/*********************************************************************/ 

Rel Rel_Teilpunkt(double* a, double* b)
{
  if( (*a) > (*b) )
    return Greater;
  else
    return Less;
}



/*********************************************************************/
/* void TeileStaebe(void)                                            */
/*********************************************************************/ 
/*                                              
   alle Stbe des Kernmodells werden in 'anz' Teile geteilt
   jh, 03.05.97
*/
/*********************************************************************/ 

void TeileStaebe_Km(void)
{
  KeGlied *tmp_kette;
  TypStab_Km *tmp_stab_Km;
  KeGlied *p;
  KeGlied *l;

  /* ungeteilte Stbe zwischenspeichern */
  tmp_kette = KeKetteDefinieren();

  /* Schleife ber alle Stbe im Km */
  KeDurchlaufVorwaerts(p,Modell->KeStab_Km) {
    tmp_stab_Km = p->x; 
    KeElementEintragen(tmp_stab_Km, Nach, tmp_kette);
  }   

  /* Stbe teilen */
  KeDurchlaufVorwaerts(p,tmp_kette) {
    tmp_stab_Km = p->x; 
    Stabteilen_Km(Modell->prozessor_anz, tmp_stab_Km);   
  }
  
  /* temp. Kette lschen */
  while(tmp_kette->nach != tmp_kette) {	  
    l = tmp_kette->nach;
    KeGliedAustragen(l);          
  } 

}
