/*
  Datei     : Zeichne_Lastedit.c
  erstellt  : 16.03.97
  geaendert : 20.03.97
  Autor     : jh
*/


#include "XStabDek.h"



/*********************************************************************/
/* void Zeichne_Lastedit(void)                                       */
/*********************************************************************/ 
/*                                              
   Funktion zeichnet Stab, Koordinatensystem und
   angewhlte Last im Lastedit-Fenster 
   jh, 16.03.97
*/
/*********************************************************************/

void Zeichne_Lastedit(void)
{
  FL_Coord win_h,win_w;
  long dummy,id;
  
  TypStab_Nm *tmp_stab;
  TypKnotenlast_Nm *tmp_pl;
  TypStreckenlast_Nm *tmp_sl; 

  /* Bestimme Fenstergre */
  fl_get_winsize(lasteditwin, &win_w, &win_h);

  /* Hole selektierten Stab */
  tmp_stab = Suche_Stab_select_Nm(&dummy);  

  /* Hole ID der Last, die im Brow2 angewhlt ist */
  id = HoleLast_ID_Lastedit();

  if(tmp_stab)  {
    XClearWindow(display,lasteditwin);
    XSetWindowBackground( display, lasteditwin, xs_color.bg );

    Zeichne_KoordSys(lasteditwin, tmp_stab);
    Zeichne_Lastedit_Stab((long) win_w, (long) win_h,tmp_stab);     
      
    if(id == -1)	
      return;     
    else {
      /* Streckenlast */	  
      if((tmp_sl = Suche_Last_SL_Nm(id)))  {
	Zeichne_Lastedit_Streckenlast((long) win_w, (long) win_h, tmp_stab, tmp_sl);
	return;
      }
      
      /* Punktlast */     
      if((tmp_pl = Suche_Last_PL_Nm(id)))	    
	Zeichne_Lastedit_Knotenlast((long) win_w, (long) win_h, tmp_stab, tmp_pl);
    }  
  }
}



/***********************************************************/ 
/* void Zeichne_Lastedit_Stab(void)                        */
/***********************************************************/ 
/*                                              
   Funktion zeichnet Stab im Lastedit-Fenster 
   jh, 16.03.97
*/
/***********************************************************/

void Zeichne_Lastedit_Stab(long win_w, long win_h, TypStab_Nm *tmp_stab)
{
  double xa,ya,xb,yb;  
  TypPunkt *tmp_punkt1,*tmp_punkt2;
  double mass;
  double s,c,abstand = 4;
  double off_x,off_y;

  int line_style = LineSolid;
  int line_width = 2;
  int cap_style = CapButt;
  int join_style = JoinMiter;
 
  XSetLineAttributes(display,gc,line_width,line_style,cap_style,join_style);

  XSetForeground(display,gc, xs_color.stab);
  XSetBackground(display,gc, xs_color.bg);

  s =  (*tmp_stab).d_sin;
  c =  (*tmp_stab).d_cos;

  tmp_punkt1 = (*tmp_stab).pe_punkta;
  tmp_punkt2 = (*tmp_stab).pe_punktb;
  
  xa =  (*tmp_punkt1).d_x;
  ya =  (*tmp_punkt1).d_y;
  xb =  (*tmp_punkt2).d_x;
  yb =  (*tmp_punkt2).d_y;

  BerechneMass(&mass, &off_x, &off_y, win_w, win_h, tmp_stab);
  
  /* Stab zeichnen */
  XDrawLine(display,lasteditwin,gc,
	    (long) (xa * mass + off_x ), (long) ( win_h - ya * mass + off_y),
	    (long) (xb * mass + off_x ), (long) ( win_h - yb * mass + off_y) );     

  /* Bezugsfaser zeichnen */
  line_style = LineOnOffDash;
  XSetLineAttributes(display,gc,line_width,line_style,cap_style,join_style);

  XDrawLine ( display,lasteditwin,gc,
	      (long) (xa * mass + off_x  + (abstand * s) ), 
	      (long) ( win_h - ya * mass + off_y + (abstand * c) ),
	      (long) (xb * mass + off_x  + (abstand * s) ),
	      (long) ( win_h - yb * mass + off_y + (abstand * c) )  );  

  /* Punkt A und B beschriften */
  XSetFont(display,gc,font_large);
  XSetForeground(display,gc,xs_color.punkt_nr);
  
  XDrawImageString(display,lasteditwin,gc, 
		   (long) (xa * mass + off_x - 20), 
		   (long) ( win_h - ya * mass + off_y + 20 ),
		   "A",1);

  XDrawImageString(display,lasteditwin,gc, 
		   (long) (xb * mass + off_x + 10), 
		   (long) ( win_h - yb * mass + off_y - 10),
		   "B",1);      

  XSetFont(display,gc,font_medium);

  /* Punkt A und B zeichnen */
  {
    Typ_DsScale m;

    XSetForeground(display,gc,xs_color.punkt);

    /* Mastab setzen */
    m.system = mass;
    m.off_x = off_x; m.off_y = off_y;
    m.win = lasteditwin;
    m.display = display;
    m.gc = gc;         
    
    ds_punkt(xa,ya,m);	
    ds_punkt(xb,yb,m);
  }

}      



/***********************************************************/ 
/* void Zeichne_Lastedit_Knotenlast(...)                   */
/***********************************************************/ 
/*                                              
   Funktion zeichnet Freiheitsgrade des Stabes
   im Fenster Lastedit
   jh, 20.03.97
*/
/***********************************************************/
  
void Zeichne_Lastedit_FreiGrad(long win_w, long win_h, 
			       TypStab_Nm *tmp_stab)
{ 
  double mass;
  double off_x,off_y; 

  long i;
  double xa,ya,xe,ye;
  double x1,x2,y1,y2;
  double alpha;
  TypPunkt *tmp_punkt;
  TypFreiGrad *tmp_freigrad;
  double abstand;
  double ab=.4;
  double laenge=40;
  char c_nummer[10]; 

  int line_style = LineSolid;
  int line_width = 2;
  int cap_style = CapButt;
  int join_style = JoinMiter;

  XSetForeground(display,gc,fl_get_flcolor(FL_DARKTOMATO));
  XSetLineAttributes(display,gc,line_width,line_style,cap_style,join_style);  

  BerechneMass( &mass, &off_x, &off_y,win_w,win_h,tmp_stab);

  /* Hole Kordinaten */
  tmp_punkt = (*tmp_stab).pe_punkta;       
  xa = (*tmp_punkt).d_x;
  ya = (*tmp_punkt).d_y;    

  tmp_punkt = (*tmp_stab).pe_punktb;
  xe = (*tmp_punkt).d_x;
  ye = (*tmp_punkt).d_y;    
  
  /* Zeichne 6 Freiheitsgrade */
  abstand = ab;
  for(i=0;i<6;i++)
    {
      tmp_freigrad = (*tmp_stab).pe_frei[i];
      
      if(i>2)
	abstand = 1.8 - ab;
      
      alpha =  (*tmp_freigrad).d_alpha;
      
      switch ( (*tmp_freigrad).l_art)
	{	      
	case 0 :
	  
	  x1 = 0.5 * (xa * (2 - abstand) + xe * abstand);
	  y1 = 0.5 * (ya * (2 - abstand) + ye * abstand); 
	  
	  x1 = (x1 * mass + off_x );
	  x2 = x1 + laenge * cos(alpha);
	  
	  y1 = (win_h - y1 * mass + off_y);
	  y2 = y1 - laenge * sin(alpha);
	  
	  ZeichnePfeil(x1,y1,x2,y2,lasteditwin);	    	    	     
	  
	  sprintf(c_nummer,"%ld",  (*tmp_freigrad).l_nummer );     
	  
	  XDrawImageString(display,lasteditwin,gc,
			   (long) (x2 + 5),
			   (long) y2,
			   c_nummer,strlen(c_nummer));     
	  break;
	  
	case 1 :
	  
	  x1 = 0.5 * (xa * (2 - abstand) + xe * abstand);
	  y1 = 0.5 * (ya * (2 - abstand) + ye * abstand); 
	  
	  x1 = (x1 * mass + off_x );
	  x2 = x1 - laenge * sin(alpha);
	  
	  y1 = (win_h - y1 * mass + off_y);
	  y2 = y1 - laenge * cos(alpha);
	  
	  ZeichnePfeil(x1,y1,x2,y2,lasteditwin);
	  
	  sprintf(c_nummer,"%ld",  (*tmp_freigrad).l_nummer );     
	  
	  XDrawImageString(display,lasteditwin,gc,
			   (long) x2,
			   (long) (y2 - 5),
			   c_nummer,strlen(c_nummer));     
	  break;
	  
	case 2 :
	  
	  x1 = 0.5 * (xa * (2 - abstand) + xe * abstand);
	  y1 = 0.5 * (ya * (2 - abstand) + ye * abstand); 	      	  
	  
	  XDrawArc(display,lasteditwin,gc,
		   (long) ((x1 * mass + off_x) - (laenge-(laenge/5))),
		   (long) ((win_h - y1 * mass + off_y) - (laenge-(laenge/5))),
		   (long) (2 * ( laenge - (laenge / 5))),
		   (long) (2 * ( laenge - (laenge / 5))),
		   (long) (20*64),
		   (long) (50 * 64) );
	  
	  sprintf(c_nummer,"%ld",  (*tmp_freigrad).l_nummer );     
	  
	  XDrawImageString(display,lasteditwin,gc,
			   (long) ( x1 * mass + off_x + laenge/1.3),
			   (long) (win_h - y1 * mass + off_y - laenge/1.3),
			   c_nummer,strlen(c_nummer));     
	  break;
	  
	default :
	  printf("Fehler bei FreiGrad - Art !\n");
	}
    }            
}    



/***********************************************************/ 
/* void Zeichne_Lastedit_Knotenlast(...)                   */
/***********************************************************/ 
/*                                              
   Funktion zeichnet eine Punktlast auf den Stab
   im Fenster Lastedit
   jh, 17.03.97
*/
/***********************************************************/

void Zeichne_Lastedit_Knotenlast(long win_w, long win_h, 
				 TypStab_Nm *tmp_stab, 
				 TypKnotenlast_Nm *tmp_pl)
{
  double mass,off_x,off_y;
  double hilf;
  double x,y;
  double alpha,kraft,ab;
  double mass_kraft;
  char *c_mass_kraft; 

  int line_style = LineSolid;
  int line_width = 2;
  int cap_style = CapButt;
  int join_style = JoinMiter;

  TypPunkt *tmp_punkt1,*tmp_punkt2;  


  XSetForeground(display,gc,xs_color.punktlast);
  XSetLineAttributes(display,gc,line_width,line_style,cap_style,join_style);

  /* Hole Mastab fr Einzellast */
  c_mass_kraft = (char*) fl_get_input(scale_form->input4); 
  mass_kraft = atof(c_mass_kraft);

  /* Mastab vergrern */
  mass_kraft *= 10;

  /* Berechne Mastab fr Darstellung des Stabes */
  BerechneMass( &mass, &off_x, &off_y,win_w,win_h,tmp_stab);          

  /* Berechne Koordinaten des Angriffpunktes */
  tmp_punkt1 = (*tmp_stab).pe_punkta;
  tmp_punkt2 = (*tmp_stab).pe_punktb;
  
  x = (*tmp_punkt1).d_x * (1-(*tmp_pl).d_pos) 
    + (*tmp_punkt2).d_x *    (*tmp_pl).d_pos;

  y = (*tmp_punkt1).d_y * (1-(*tmp_pl).d_pos) 
    + (*tmp_punkt2).d_y *    (*tmp_pl).d_pos;
 
  kraft = (*tmp_pl).d_last;
   
  /* zeichne Moment */
  if( (*tmp_pl).l_art == 2) 
    {
      if( kraft<0 )
	ab = 1.2;
      else
	ab = -1.2;		     			

      /* Mastab fr Moment verkleinern */
      mass_kraft *= 0.2;

      /* Angriffspunkt zeichnen */
      XDrawArc(display,lasteditwin,gc,
	       (long) ((x * mass + off_x) - 3),
	       (long) ((win_h - y * mass + off_y) - 3),
	       (long) 6,
	       (long) 6,
	       0L,
	       23040L);

      /* Pfeil zeichnen */
      XDrawArc(display,lasteditwin,gc,
	       (long) ((x * mass + off_x) - 30),
	       (long) ((win_h - y * mass + off_y) - 30),
	       (long) 60,
	       (long) 60,
	       0L,
	       (long) (kraft*mass_kraft*640) );
      XDrawLine(display,lasteditwin,gc,
		(long) ((x * mass + off_x) + 30 * cos(kraft*mass_kraft*PI/18)),
		(long) ((win_h - y * mass + off_y) - 30 * sin(kraft*mass_kraft*PI/18)),
		(long) ((x * mass + off_x) + 25 * cos((kraft*mass_kraft+ab)*PI/18)),
		(long) ((win_h - y * mass + off_y) - 25 * sin((kraft*mass_kraft+ab)*PI/18)));
      XDrawLine(display,lasteditwin,gc,
		(long) ((x * mass + off_x) + 30 * cos(kraft*mass_kraft*PI/18)),
		(long) ((win_h - y * mass + off_y) - 30 * sin(kraft*mass_kraft*PI/18)),
		(long) ((x * mass + off_x) + 35 * cos((kraft*mass_kraft+ab)*PI/18)),
		(long) ((win_h - y * mass + off_y) - 35 * sin((kraft*mass_kraft+ab)*PI/18))); 	
      return;
    }


  /* zeichne Einzelkraft */  
  if((*tmp_pl).l_art == 1) 
    {
      if((*tmp_pl).l_koordsys_winkel == 2) /* global */
	hilf = 0;
      else
	hilf = 1;
      
      alpha = (*tmp_pl).d_alpha + ((*tmp_stab).d_alpha * hilf);
      
      ZeichnePfeil((long) ( (x * mass + off_x) - (kraft * sin(alpha) * mass_kraft) ), 
		   (long) ( (win_h - y * mass + off_y) - (kraft * cos(alpha) * mass_kraft) ),
		   (long) (x * mass + off_x) , 
		   (long) (win_h - y * mass + off_y),
		   lasteditwin);       
    } 
}



/*********************************************************************/
/* void Zeichne_Lastedit_Streckenlast( ... )                         */
/*********************************************************************/ 
/*                                              
   Funktion zeichnet eine Streckenlast auf den Stab
   im Fenster Lastedit
   jh, 20.03.97
*/
/*********************************************************************/

void Zeichne_Lastedit_Streckenlast(long win_w, long win_h, 
				   TypStab_Nm *tmp_stab, 
				   TypStreckenlast_Nm *tmp_sl)
{ 
  double mass,off_x,off_y;
  long i,anz=10;
  long hilf;
  double xa,ya,xe,ye;
  double x1,y1,x2,y2;
  double z,z2;
  double step,step2;
  double La,Le,L;
  double s,c;
  double abstand,ab = 5;

  double alpha; 
  double mass_kraft;
  char *c_mass_kraft; 

  int line_style = LineSolid;
  int line_width = 2;
  int cap_style = CapButt;
  int join_style = JoinMiter;

  TypPunkt *tmp_punkt1,*tmp_punkt2;
 
  XSetForeground(display,gc,xs_color.streckenlast);
  XSetLineAttributes(display,gc,line_width,line_style,cap_style,join_style); 

  /* hole Mastab */
  c_mass_kraft = (char*) fl_get_input(scale_form->input4); 
  mass_kraft = atof(c_mass_kraft)/2;

  BerechneMass( &mass, &off_x, &off_y,win_w,win_h,tmp_stab);

  s =  (*tmp_stab).d_sin;
  c =  (*tmp_stab).d_cos;            

  tmp_punkt1 = (*tmp_stab).pe_punkta;
  tmp_punkt2 = (*tmp_stab).pe_punktb;
  
  xa = (*tmp_punkt1).d_x;
  ya = (*tmp_punkt1).d_y;
  xe = (*tmp_punkt2).d_x;
  ye = (*tmp_punkt2).d_y;  
  
  La = (*tmp_sl).d_lasta;
  Le = (*tmp_sl).d_lastb;	    	    	    
	
  if((*tmp_sl).l_koordsys == 2) /* global */
    hilf = 0;
  else
    hilf = 1;

  alpha = (*tmp_sl).d_alpha + ((*tmp_stab).d_alpha * hilf);

  step =  (double) (((*tmp_sl).d_end - (*tmp_sl).d_anf) / (anz-1));
  step2 = (double) (1. / (anz-1));

  z = (*tmp_sl).d_anf;
  z2 = 0;

  for(i=0;i<anz;i++)  {      
    x1 = xa*(1-z) + xe * z;
    y1 = ya*(1-z) + ye * z;
    
    L = ( La*(1-z2) + Le * z2 ) * mass_kraft;
    
    x2 = x1 - L * sin(alpha);
    y2 = y1 + L * cos(alpha);
    
    if(L>0)
      abstand = ab;
    else
      abstand = -ab;
    
    ZeichnePfeil((long) (x2 * mass + off_x - (abstand * s) ),
		 (long) (win_h - y2 * mass + off_y - (abstand *c)),
		 (long) (x1 * mass + off_x - (abstand * s) ) , 
		 (long) (win_h - y1 * mass + off_y - (abstand *c)),
		 lasteditwin);			       
    z+=step;
    z2+=step2;
  }
}


