/*
  Datei     : zeichne_verschiebung.c
  erstellt  : 03.04.97
  gendert  : 03.05.97
  Autor     : jh
  Bemerkung : 
*/


#include "XStabDek.h"





/*********************************************************************/
/*   void Zeichne(void)                                              */
/*********************************************************************/ 
/*                                              
   Fkt zeichnet das verschobene System im Hauptfenster 
   der grafischen Ausgabe  
   jh, 03.04.97

*/
/*********************************************************************/

void Zeichne_Verschiebung(TypMaszstab mass)
{       
  KeGlied *intervalle;
  KeGlied *inter;
  TypIntervall *tmp_intervall;
  
  KeGlied *p;
  TypStab_Nm *tmp_stab;

  int line_style = LineSolid;
  int line_width ;
  int cap_style = CapButt;
  int join_style = JoinMiter; 
  
  
  XSetForeground(display,gc,xs_color.verschiebung); 

  if(!Modell->KeStab_Nm)
    return;

  KeDurchlaufVorwaerts(p,Modell->KeStab_Nm)  {
    tmp_stab = p->x;     
    intervalle = (*tmp_stab).intervalle;  
         
    if(!intervalle)
      continue;

    XSetLineAttributes(display,gc,line_width=3,line_style,cap_style,join_style);

    /* Schleife ber alle Intervalle eines Stabes */
    KeDurchlaufVorwaerts(inter, intervalle)  { 
      tmp_intervall = inter->x;
      Zeichne_Verschiebung_Stab(win, tmp_intervall, mass);
    }

    if(fl_get_button(drawcontrol_system_form->verschiebung_beschriften_button)) {
      XSetLineAttributes(display,gc,line_width=0,line_style,cap_style,join_style);
      Zeichne_Beschriftung_Verschiebung(win, tmp_stab, mass);
    }
    
  } 

}



/*********************************************************************/
/*   void Zeichne(void)                                              */
/*********************************************************************/ 
/*                                              
   Fkt zeichnet die Verschiebung eines Intervalles  
   in einem bestimmten Fenster 
   jh, 03.05.97
*/
/*********************************************************************/

void Zeichne_Verschiebung_Stab(Window win, 
			       TypIntervall *tmp_intervall,
			       TypMaszstab mass)
{ 
  long i;
  double z,step,x,y;
  double xa,ya,xb,yb;
  double dx,dy;
  double vxa,vya,vxb,vyb,vza,vzb;
  double vs[6];
  long anz_points = 5;
  double alpha1=0,alpha2=0;
  double alpha_stab; 
  double s,c;

  XPoint *points;  

  points = (XPoint*) calloc ( anz_points , sizeof(XPoint));  
	  
  for(i=0;i<6;i++)	    
    vs[i] = (*tmp_intervall).d_vs[i];
  
  alpha_stab = (*tmp_intervall).d_alpha_stab;
  c = cos(alpha_stab);
  s = sin(alpha_stab);

  /* Drehung der Freiheitsgrade */
  alpha1 = (*tmp_intervall).d_alpha1;	     
  alpha2 = (*tmp_intervall).d_alpha2;	   
  
  /* Anfangs- und Endkoordinaten des Intervalles */
  xa = (*tmp_intervall).d_xa;   ya = (*tmp_intervall).d_ya;  
  xb = (*tmp_intervall).d_xe;   yb = (*tmp_intervall).d_ye;
    

  /* Verschiebungen im globalen Koordinatensystem */
  vxa = mass.vs * ( cos(alpha_stab - alpha1) * vs[0] + sin(alpha_stab - alpha1) * vs[1]);
  vya = mass.vs * ( -sin(alpha_stab - alpha1) * vs[0] + cos(alpha_stab - alpha1) * vs[1]);
  
  vxb = mass.vs * ( cos(alpha_stab - alpha2) * vs[3] + sin(alpha_stab - alpha2) * vs[4]);
  vyb = mass.vs * ( -sin(alpha_stab - alpha2) * vs[3] + cos(alpha_stab - alpha2) * vs[4]);
  
  vza = mass.vs * vs[2];
  vzb = mass.vs * vs[5];      
  
  step = (double) (2./((double) anz_points - 1.));
  
  z=-1.;
  
  for(i=0;i<anz_points;i++) {	

    /* lineare Interpolation der Verschiebung in Stabachse */
    dx = 0.5 * (vxa * (1.-z) + (vxb+(*tmp_intervall).d_laenge) * (1.+z));

    /* Hermite - Interpolation w,u im glob. Koord.sys */
    dy = Hermite_dis(vya, vza, vyb, vzb,
		     (*tmp_intervall).d_laenge,z);
    
    x = xa + (c * dx - s * dy);
    y = ya + (s * dx + c * dy );
    
    points[i].x = (long) (X_welt_win_m(win,x,mass));
    points[i].y = (long) (Y_welt_win_m(win,y,mass));
    
    z+=step;
    
  }    
  
  XDrawLines(display, win, gc, points, anz_points, CoordModeOrigin);

  free(points);

}



/*********************************************************************/
/* void Zeichne_Beschriftung_Verschiebung( ... )                     */
/*********************************************************************/
/*                                              
   Funktion beschriftet die Verschiebung 
   jh, 19.04.97 
*/
/*********************************************************************/

void Zeichne_Beschriftung_Verschiebung(Window win,
				       TypStab_Nm* tmp_stab,
				       TypMaszstab mass)
{ 
  double W,U,pos=0;
  TypPunkt *pA,*pB;
  double x0,y0,x1,y1,dx,dy;
  char c_text_u[20];
  char c_text_w[20];

  if(!(*tmp_stab).intervalle)
    return;

  /* Geometrie interpolieren */
  pA = (*tmp_stab).pe_punkta;
  pB = (*tmp_stab).pe_punktb;


  /* Pos 0. und 1. zeichnen */
  pos = 0.;
  while( pos <= 1. ) {                

    W = get_w(tmp_stab,pos);   
    U = get_u(tmp_stab,pos);
    sprintf(c_text_w,"w:%.4f",W);
    sprintf(c_text_u,"u:%.4f",U);  
      
    x0 = (*pA).d_x * (1-pos) + (*pB).d_x * pos;
    y0 = (*pA).d_y * (1-pos) + (*pB).d_y * pos;
    
    dx = (W * sin((*tmp_stab).d_alpha) + U * cos((*tmp_stab).d_alpha)) * mass.vs;
    dy = (U * sin((*tmp_stab).d_alpha) - W * cos((*tmp_stab).d_alpha)) * mass.vs;
    
    x1 = x0 + dx;
    y1 = y0 + dy;	            	            
    
    XDrawLine(display,win,gc, 
	      (long) X_welt_win_m(win,x1,mass),
	      (long) Y_welt_win_m(win,y1,mass),
	      (long) (X_welt_win_m(win,x1,mass) + 30 - (100 * pos)),
	      (long) (Y_welt_win_m(win,y1,mass) - 30));
    
    XDrawImageString(display,win,gc,
		     (long) (X_welt_win_m(win,x1,mass) + 30 - (100 * pos)),
		     (long) (Y_welt_win_m(win,y1,mass) - 30),
		     c_text_w,strlen(c_text_w));     

    XDrawImageString(display,win,gc,
		     (long) (X_welt_win_m(win,x1,mass) + 30 - (100 * pos)),
		     (long) (Y_welt_win_m(win,y1,mass) - 15),
		     c_text_u,strlen(c_text_u));   
    
    pos += 1.;
  }


  /* Maximum beschriften */
  
  W = max_w(tmp_stab,&pos); 

  if( pos > XS_LIMIT &&  pos < (1-XS_LIMIT)) {
    U = get_u(tmp_stab,pos);
    sprintf(c_text_w,"w:%.4f",W);
    sprintf(c_text_u,"u:%.4f",U);  
    
    x0 = (*pA).d_x * (1-pos) + (*pB).d_x * pos;
    y0 = (*pA).d_y * (1-pos) + (*pB).d_y * pos;
    
    dx = (W * sin((*tmp_stab).d_alpha) + U * cos((*tmp_stab).d_alpha)) * mass.vs;
    dy = (U * sin((*tmp_stab).d_alpha) - W * cos((*tmp_stab).d_alpha)) * mass.vs;
    
    x1 = x0 + dx;
    y1 = y0 + dy;	            	            
    
    XDrawLine(display,win,gc, 
	      (long) X_welt_win_m(win,x1,mass),
	      (long) Y_welt_win_m(win,y1,mass),
	      (long) (X_welt_win_m(win,x1,mass) + 30),
	      (long) (Y_welt_win_m(win,y1,mass) - 30));
    
    XDrawImageString(display,win,gc,
		     (long) (X_welt_win_m(win,x1,mass) + 30),
		     (long) (Y_welt_win_m(win,y1,mass) - 30),
		     c_text_w,strlen(c_text_w));     
    
    XDrawImageString(display,win,gc,
		     (long) (X_welt_win_m(win,x1,mass) + 30),
		     (long) (Y_welt_win_m(win,y1,mass) - 15),
		     c_text_u,strlen(c_text_u));   
  }
    
    
  /* Minimum beschriften */

  W = min_w(tmp_stab,&pos); 

  if( pos > XS_LIMIT &&  pos < (1-XS_LIMIT)) {
    U = get_u(tmp_stab,pos);
    sprintf(c_text_w,"w:%.4f",W);
    sprintf(c_text_u,"u:%.4f",U);  
    
    x0 = (*pA).d_x * (1-pos) + (*pB).d_x * pos;
    y0 = (*pA).d_y * (1-pos) + (*pB).d_y * pos;
    
    dx = (W * sin((*tmp_stab).d_alpha) + U * cos((*tmp_stab).d_alpha)) * mass.vs;
    dy = (U * sin((*tmp_stab).d_alpha) - W * cos((*tmp_stab).d_alpha)) * mass.vs;
    
    x1 = x0 + dx;
    y1 = y0 + dy;	            	            
    
    XDrawLine(display,win,gc, 
	      (long) X_welt_win_m(win,x1,mass),
	      (long) Y_welt_win_m(win,y1,mass),
	      (long) (X_welt_win_m(win,x1,mass) + 30),
	      (long) (Y_welt_win_m(win,y1,mass) - 30));
    
    XDrawImageString(display,win,gc,
		     (long) (X_welt_win_m(win,x1,mass) + 30),
		     (long) (Y_welt_win_m(win,y1,mass) - 30),
		     c_text_w,strlen(c_text_w));     
    
    XDrawImageString(display,win,gc,
		     (long) (X_welt_win_m(win,x1,mass) + 30),
		     (long) (Y_welt_win_m(win,y1,mass) - 15),
		     c_text_u,strlen(c_text_u));   
  }
  
}


